#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : yaml
# @Time         : 2019-07-10 14:51
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import yaml
from pprint import pprint

"""
https://www.cnblogs.com/klb561/p/9326677.html
https://blog.csdn.net/zhusongziye/article/details/80024426
"""

# dic = yaml.safe_load(open('yaml.yaml'))

# for k, v in dic.items():
#     print(k, ":", type(v).__name__, ':', v)

dic = yaml.full_load(open('yaml_demo.yaml'))

pprint(dic)

yaml.dump(dic, open('yaml_temp.yml', 'w'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : SavedModel
# @Time         : 2019-06-24 16:24
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


# path = '/tmp/simple_keras_model'
# tf.keras.experimental.export_saved_model(model, path)
#
# # Load the saved keras model back.
# new_model = tf.keras.experimental.load_from_saved_model(path)
# new_model.summary()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : metrics
# @Time         : 2019-07-03 17:10
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import tensorflow as tf
import tensorflow.python.keras.backend as K
tf.keras.metrics.Accuracy()

# 参考losses.py编写自己的损失函数
# Retinanet 针对正负样本不均衡问题设计的损失函数focal_loss
# Keras 以tf为后端；可以使用Keras的math方法或者tf的math方法写都可以keras被识别
def focal_loss(y_true, y_pred, gamma=2., alpha=.25):
    pt_1 = tf.where(tf.equal(y_true, 1), y_pred, tf.ones_like(y_pred))
    pt_0 = tf.where(tf.equal(y_true, 0), y_pred, tf.zeros_like(y_pred))
    return K.mean(-alpha * K.pow(1 - pt_1, gamma) * K.log(pt_1) - (1 - alpha) * K.pow(pt_0, gamma) * K.log(1 - pt_0),
                  axis=-1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : xgboost_gpu
# @Time         : 2019-07-03 14:31
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

"""
xgb/lgb 参数https://www.jianshu.com/p/1100e333fcab
"""
from xgboost import XGBClassifier
from sklearn.datasets import make_classification
from sklearn.model_selection import train_test_split

X, y = make_classification(100000)
X_train, X_test, y_train, y_test = train_test_split(
    X, y, test_size=0.33, random_state=42)


clf = XGBClassifier(learning_rate=0.01,
                    n_estimators=10000,
                    tree_method='gpu_hist',
                    predictor='gpu_predictor',
                    verbosity=0,
                    n_jobs=8,
                   )

clf.fit(X, y,
        eval_metric='auc',
        eval_set=[(X_train, y_train), (X_test, y_test)],
        early_stopping_rounds=300,
        verbose=100)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : embedding
# @Time         : 2019-07-22 11:53
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import matplotlib.pyplot as plt
from sklearn.manifold import TSNE
from gensim.models.fasttext import load_facebook_model

fasttext = load_facebook_model('../../../fds/data/wv/skipgram.title')

word = '周杰伦'
words = [word]
words_v = [fasttext.wv[word]]
for w, _ in fasttext.wv.similar_by_word('周杰伦', 50):
    words.append(w)
    words_v.append(fasttext.wv[w])

points = TSNE().fit_transform(words_v)

plt.figure(figsize=(12, 12))  # in inches
for i in range(len(words)):
    x = words_v[i][0]
    y = words_v[i][1]
    plt.scatter(x, y)
    plt.annotate(words[i], (x, y), ha='center', va='top')
plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'Sent2Vec'
__author__ = 'JieYuan'
__mtime__ = '19-3-1'
"""

import numpy as np
from tqdm import tqdm

"""
1. 加和/平均
2. Concatenated p-mean Word Embeddings: https://blog.csdn.net/triplemeng/article/details/81298100
3. InferSent: https://github.com/facebookresearch/InferSent
"""


class Sent2Vec(object):

    def __init__(self, gensim_model=None, fname=None):
        gensim_model.init_sims(replace=True)  # 节省内存
        self.gensim_model = gensim_model
        self.fname = fname
        self._load_wv()

    def transform(self, sentence, tokenizer=str.split, mode='mean', normalize=False):
        words = []
        flag = isinstance(self.embeddings, dict)
        for w in tokenizer(sentence.lower()):
            if flag and w in self.embeddings:
                w = self.embeddings[w]
            else:
                w = self.embeddings[w]
            words.append(w)
        if words:
            if mode in ('sum', 'mean'):
                _ = np.array(words)
                _ = _.__getattribute__(mode)(axis=0)
                return _ / np.sqrt(np.clip((_ ** 2).sum(), 1e-12, None)) if normalize else _
            else:
                return np.zeros(self.word_size)
        else:
            return np.zeros(self.word_size)

    def _load_wv(self):
        if self.fname is None and self.gensim_model is None:
            raise Exception('fname and gensim_model can not be both None!')

        if self.gensim_model:
            self.word_size = self.gensim_model.wv.vector_size
            self.embeddings = self.gensim_model.wv

        if self.fname:
            with open(self.fname) as f:
                for line in tqdm(f, 'Load Vectors ...'):
                    line = line.strip().split()
                    if len(line) > 2:
                        self.embeddings[line[0]] = np.asarray(line[1:], dtype='float32')
                self.word_size = len(line[1:])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : colors
# @Time         : 2019-07-04 14:03
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


class Colors:
    ok = '\033[92m'
    fail = '\033[91m'
    close = '\033[0m'



import csv

print(csv.QUOTE_NONE)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : capsule
# @Time         : 2019-07-10 16:41
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from tensorflow.python.keras import utils
from tensorflow.python.keras.datasets import mnist
from tensorflow.python.keras.models import Model
from tensorflow.python.keras.layers import *
from tensorflow.python.keras import backend as K
import numpy as np
from tql.algo_dl.keras.models.Capsule import Capsule

# 准备训练数据
batch_size = 128
num_classes = 10
img_rows, img_cols = 28, 28

(x_train, y_train), (x_test, y_test) = mnist.load_data() # '../../../fds/data/mnist.npz'
x_train = x_train.reshape(x_train.shape[0], img_rows, img_cols, 1)
x_test = x_test.reshape(x_test.shape[0], img_rows, img_cols, 1)

x_train = x_train.astype('float32')
x_test = x_test.astype('float32')
x_train /= 255
x_test /= 255
y_train = utils.to_categorical(y_train, num_classes)
y_test = utils.to_categorical(y_test, num_classes)

# 准备自定义的测试样本
# 对测试集重新排序并拼接到原来测试集，就构成了新的测试集，每张图片有两个不同数字
idx = list(range(len(x_test)))
np.random.shuffle(idx)
X_test = np.concatenate([x_test, x_test[idx]], 1)
Y_test = np.vstack([y_test.argmax(1), y_test[idx].argmax(1)]).T
X_test = X_test[Y_test[:, 0] != Y_test[:, 1]]  # 确保两个数字不一样
Y_test = Y_test[Y_test[:, 0] != Y_test[:, 1]]
Y_test.sort(axis=1)  # 排一下序，因为只比较集合，不比较顺序

# 搭建CNN+Capsule分类模型
input_image = Input(shape=(None, None, 1))
cnn = Conv2D(64, (3, 3), activation='relu')(input_image)
cnn = Conv2D(64, (3, 3), activation='relu')(cnn)
cnn = AveragePooling2D((2, 2))(cnn)
cnn = Conv2D(128, (3, 3), activation='relu')(cnn)
cnn = Conv2D(128, (3, 3), activation='relu')(cnn)
cnn = Reshape((-1, 128))(cnn)
capsule = Capsule(10, 16, 3, True)(cnn)
output = Lambda(lambda x: K.sqrt(K.sum(K.square(x), 2)), output_shape=(10,))(capsule)

model = Model(inputs=input_image, outputs=output)
model.compile(
    loss=lambda y_true, y_pred: y_true * K.relu(0.9 - y_pred) ** 2 + 0.25 * (1 - y_true) * K.relu(y_pred - 0.1) ** 2,
    optimizer='adam',
    metrics=['accuracy'])

model.summary()

model.fit(x_train, y_train,
          batch_size=batch_size,
          epochs=10,
          verbose=1,
          validation_data=(x_test, y_test))

Y_pred = model.predict(X_test)  # 用模型进行预测
greater = np.sort(Y_pred, axis=1)[:, -2] > 0.5  # 判断预测结果是否大于0.5
Y_pred = Y_pred.argsort()[:, -2:]  # 取最高分数的两个类别
Y_pred.sort(axis=1)  # 排序，因为只比较集合

acc = 1.*(np.prod(Y_pred == Y_test, axis=1)).sum()/len(X_test)
print ('CNN+Capsule，不考虑置信度的准确率为：%s'%acc)
acc = 1.*(np.prod(Y_pred == Y_test, axis=1)*greater).sum()/len(X_test)
print ('CNN+Capsule，考虑置信度的准确率为：%s'%acc)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-1-2'
"""

import time
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from sklearn.model_selection import StratifiedKFold, RepeatedStratifiedKFold
from sklearn.metrics import roc_auc_score, mean_squared_error

from lightgbm import LGBMClassifier
from xgboost import XGBClassifier
from catboost import CatBoostClassifier
from statsmodels.api import GLM, families


class OOF(object):
    """Out of flod prediction
    # TODO 支持回归

    lightGBM一个一个地建立节点; XGboost一层一层地建立节点
    https://blog.csdn.net/friyal/article/details/82758532
    Catboost总是使用完全二叉树。它的节点是镜像的(对称树)。Catboost称对称树有利于避免overfit，增加可靠性，并且能大大加速预测等等。
        计算某个category出现的频率，加上超参数，生成新的numerical features
    # https://blog.csdn.net/linxid/article/details/80723811
    """
    _params = {'metric': 'auc',
               'learning_rate': 0.01,
               'n_estimators': 30000,
               'subsample': 0.8,
               'colsample_bytree': 0.8,
               'class_weight': 'balanced',  ##
               'scale_pos_weight': 1,  ##
               'random_state': 2019,
               'verbosity': -1}
    lgb = LGBMClassifier(n_jobs=16, **_params)
    xgb = XGBClassifier()
    cat = CatBoostClassifier(n_estimators=20000, learning_rate=0.05, loss_function='Logloss', eval_metric='AUC',
                             random_state=2019)

    def __init__(self, clf=None, folds=None):
        self.clf = clf if clf else self.lgb
        self.folds = folds if folds else StratifiedKFold(5, True, 2019)  # 支持 RepeatedStratifiedKFold
        self.model_type = self.clf.__repr__()
        # self.clf_agrs = self.getfullargspec(self.clf.fit).args if hasattr(self.clf, 'fit') else None

    def fit(self, X, y, X_test, feval=None, cat_feats=None, exclude_columns=None, epochs=16, batch_size=128,
            oof2csv=None):
        """
        # TODO: Rank 融合
        :param X:
        :param y:
        :param X_test:
        :param feval: roc_auc_score(y_true, y_score)
        :param cat_feats: 类别特征索引
        :param exclude_columns:
        仅针对 nn
        :param epochs:
        :param batch_size:
        :return:
        """
        # oof评估函数
        feval = feval if feval else roc_auc_score

        # 移除不需要的特征
        if exclude_columns:
            feats = X.columns.difference(exclude_columns)
        else:
            feats = X.columns

        X, X_test = X[feats], X_test[feats]

        if hasattr(self.folds, 'n_splits'):
            num_folds = self.folds.n_splits
        else:
            num_folds = self.folds.cvargs['n_splits']

        # Cross validation model
        # Create arrays and dataframes to store results
        oof_preds = np.zeros(len(X))
        sub_preds = np.zeros(len(X_test))
        self.feature_importance_df = pd.DataFrame()

        for n_fold, (train_idx, valid_idx) in enumerate(self.folds.split(X, y), 1):
            print("\n\033[94mFold %s started at %s\033[0m" % (n_fold, time.ctime()))

            X_train, y_train = X.iloc[train_idx], y.iloc[train_idx]
            X_valid, y_valid = X.iloc[valid_idx], y.iloc[valid_idx]

            if not hasattr(self.clf, 'fit'):
                print("该算法无fit方法")
                break
            else:
                if 'LGBMClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 eval_set=eval_set,
                                 categorical_feature=cat_feats if cat_feats else 'auto',
                                 eval_metric='auc',
                                 early_stopping_rounds=100,
                                 verbose=100)
                elif 'LGBMRegressor' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 eval_set=eval_set,
                                 categorical_feature=cat_feats if cat_feats else 'auto',
                                 eval_metric='l2',
                                 early_stopping_rounds=100,
                                 verbose=100)

                elif 'XGBClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 eval_set=eval_set,
                                 eval_metric='auc',
                                 early_stopping_rounds=100,
                                 verbose=100)
                elif 'XGBRegressor' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 eval_set=eval_set,
                                 eval_metric='rmse',
                                 early_stopping_rounds=100,
                                 verbose=100)

                elif 'CatBoostClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 eval_set=eval_set,
                                 cat_features=cat_feats,
                                 use_best_model=True,
                                 plot=True,
                                 early_stopping_rounds=100,
                                 verbose=100)
                elif 'CatBoostRegressor' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 eval_set=eval_set,
                                 cat_features=cat_feats,
                                 use_best_model=True,
                                 plot=True,
                                 early_stopping_rounds=100,
                                 verbose=0)

                elif 'RGFClassifier' in self.model_type:
                    pass
                elif 'RGFRegressor' in self.model_type:
                    pass

                # https://www.cnblogs.com/flyu6/p/7691106.html
                elif 'KerasClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 epochs=epochs,
                                 batch_size=batch_size,
                                 validation_data=eval_set)
                elif 'KerasRegressor' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.clf.fit(X_train, y_train,
                                 epochs=epochs,
                                 batch_size=batch_size,
                                 validation_data=eval_set)

                elif self.model_type == 'GLM':
                    # TODO: 其他模型的支持
                    self.clf = GLM(y_train, X_train, family=families.Binomial())
                    self.clf = self.clf.fit().predict(X)
                else:
                    # sklearn 原生模型
                    self.clf.fit(X, y)

                # 计算并保存 preds
                # TODO: 多分类需要修改
                if hasattr(self.clf, 'predict_proba'):
                    oof_preds[valid_idx] = self.clf.predict_proba(X_valid)[:, 1]
                    sub_preds += self.clf.predict_proba(X_test)[:, 1] / num_folds
                else:
                    oof_preds[valid_idx] = self.clf.predict(X_valid)
                    sub_preds += self.clf.predict(X_test) / num_folds

            if hasattr(self.clf, 'feature_importances_'):
                fold_importance_df = pd.DataFrame()
                fold_importance_df["feature"] = feats
                fold_importance_df["importance"] = self.clf.feature_importances_
                fold_importance_df["fold"] = n_fold
                self.feature_importance_df = pd.concat([self.feature_importance_df, fold_importance_df], 0)

        try:
            score = feval(y, oof_preds)
            score_name = feval.__repr__().split()[1]
        except Exception as e:
            score = score_name = None
            print('Error feval:', e)

        print("\n\033[94mOOF %s: %s end at %s\n\033[0m" % (score_name, score, time.ctime()))

        if hasattr(self.clf, 'feature_importances_'):
            self.plot_importances(self.feature_importance_df)

        self.oof_preds = oof_preds
        self.test_preds = sub_preds
        if oof2csv:
            pd.Series(oof_preds.tolist() + sub_preds.tolist(), name='oof').to_csv(oof2csv + time.ctime(), index=False)

        return oof_preds, sub_preds

    def plot_importances(self, df, topk=64):
        """Display/plot feature importance"""
        assert "feature" in df.columns and "importance" in df.columns, '无["feature", "importance"]'
        data = (df[["feature", "importance"]]
                .groupby("feature")
                .mean()
                .reset_index()
                .sort_values("importance", 0, False))[:topk]

        plt.figure(figsize=(12, int(topk / 4)))
        sns.barplot(x="importance", y="feature", data=data.assign(feature=data.feature.astype(str)))
        plt.title('LightGBM Features (avg over folds)')
        plt.tight_layout()
        plt.savefig('lgbm_importances.png')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-1-2'
"""
import re

chinese = re.compile('[\u4e00-\u9fa5]+')
_chinese = re.compile('[^\u4e00-\u9fa5]+')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'info_value'
__author__ = 'JieYuan'
__mtime__ = '19-3-12'
"""

from concurrent.futures import ThreadPoolExecutor

import numpy as np
import pandas as pd
from tqdm import tqdm


class InformationValue(object):
    """
    小于0.02预测能力无
    大于0.30预测能力强
    """
    def __init__(self, df: pd.DataFrame, label: str):
        """
        :param df:
        :param label: target name
        """
        assert label in df.columns

        self.label = label
        self.df = df.assign(_label=1 - df[label])
        self.feats = [col for col in df.columns if col != label]

        self.y1 = self.df[label].values.sum()
        self.y0 = self.df['_label'].values.sum()

    def iv(self, order=True, n_jobs=16):
        with ThreadPoolExecutor(max_workers=n_jobs) as pool:
            ivs = pool.map(self._iv, tqdm(self.feats, 'Calculating ...'), chunksize=1)
        z = zip(ivs, self.feats)
        return pd.DataFrame(sorted(z, reverse=True) if order else list(z), columns=['iv', 'feats'])

    def _iv(self, feat):
        gr = self.df.groupby(feat)
        gr1, gr0 = gr[self.label].sum().values + 1e-8, gr['_label'].sum().values + 1e-8
        good, bad = gr1 / self.y1, gr0 / self.y0
        woe = np.log(good / bad)
        iv = (good - bad) * woe
        return iv.sum()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : restful_sanic
# @Time         : 2019-07-18 10:13
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import jieba
from restful_api import Api
import requests

from lxml.etree import HTML
from googletrans import Translator

translator = Translator(service_urls=['translate.google.cn', 'translate.google.com'],
                        timeout=3)


def trans_google(q='苹果', fromLang='auto', toLang='en'):
    """

    :param q:
    :param fromLang:
    :param toLang: zh
    :return:
    """
    url = "http://translate.google.cn/translate_a/single?client=gtx&dt=t&dj=1&ie=UTF-8&sl=%s&tl=%s" % (fromLang, toLang)
    try:
        r = requests.get(url, {'q': q}, timeout=3)
        text = r.json()['sentences'][0]['trans']
    except Exception as e:
        print(e)
        text = translator.translate(q, toLang, fromLang).text
    return text


def get_title(url='https://baijiahao.baidu.com/s?id=1604534610481479105&wfr=spider&for=pc&isFailFlag=1'):
    r = requests.get(url, headers={'user-agent': 'Mozilla/5.0'})
    r.encoding = r.apparent_encoding
    # soup = BeautifulSoup(r.text)
    dom_tree = HTML(r.text)
    title = dom_tree.xpath('//title/text()')
    return title[0]


api = Api('/ctr/trans', trans_google, method='GET', verbose=False)
api = Api('/get_title', get_title, api.app)

api.app.run('0.0.0.0')

# import requests
# json = {'x': 1, 'y': 10}
# requests.post('http://127.0.0.1:8000/post1', json=json).json()
# requests.post('http://127.0.0.1:8000/post2', json=json).json()
# requests.post('http://127.0.0.1:8000/post3', json=json).json()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : gpu_catb
# @Time         : 2019-07-04 13:27
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from catboost import CatBoostClassifier
from sklearn.datasets import make_classification

X, y = make_classification(100000)

# object 无需数值化直接 train
# cats = X.dtypes[X.dtypes == object].index
# for i in cats:
#     X[i] = X[i].astype(str)
model = CatBoostClassifier(loss_function="Logloss",
                           eval_metric="AUC",
                           task_type="GPU",
                           learning_rate=0.05,
                           n_estimators=10000,
                           l2_leaf_reg=50,
                           random_seed=666,
                           od_type="Iter",
                           depth=5,
                           early_stopping_rounds=100,
                           border_count=64,
                           has_time=True
                           )

model.fit(X, y, eval_set=(X, y), verbose=1000, use_best_model=True, plot=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : restful
# @Time         : 2019-07-02 10:48
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import flask
import pandas as pd

import tensorflow as tf

# 实例化 flask
app = flask.Flask(__name__)


# 我们需要重新定义我们的度量函数，
# 从而在加载模型时使用它


def auc(y_true, y_pred):
    # auc = tf.metrics.auc(y_true, y_pred)[1]
    # keras.backend.get_session().run(tf.local_variables_initializer())
    return auc



# 将预测函数定义为一个端点
@app.route("/predict", methods=["GET","POST"])
def predict():
    data = {"success": False}

    params = flask.request.json
    if (params == None):
        params = flask.request.args

    # 若发现参数，则返回预测值
    if (params != None):
        x=pd.DataFrame.from_dict(params, orient='index').transpose()
        # with graph.as_default():
        #     # data["prediction"] = str(model.predict(x)[0][0])
        #     data["success"] = True

    # 返回Jason格式的响应
    return flask.jsonify(data)

# 启动Flask应用程序，允许远程连接
if __name__ == "__main__":
    app.run(host='0.0.0.0')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : _info
# @Time         : 2019-06-20 11:24
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
import os
import socket
from .. import cprint
import platform


class MyInfo(object):

    def __init__(self):
        self.hostname = socket.getfqdn(socket.gethostname())
        self.localhost = socket.gethostbyname(self.hostname)

        print(self.hostname, self.localhost)
        cprint(os.sys.version)
        cprint(platform.platform())


if __name__ == '__main__':
    MyInfo()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'plot_ys'
__author__ = 'JieYuan'
__mtime__ = '2019-04-25'
"""
import numpy as np
import pandas as pd


def plot_ys(y_true, *ys):
    ys = (y_true,) + ys
    columns = ['y%s' % i for i in range(len(ys))]
    df = (pd.DataFrame(np.column_stack(ys), columns=columns)
          .sort_values('y0')
          .reset_index(drop=True))
    df.plot(figsize=(20, 10), subplots=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : featuretools
# @Time         : 2019-07-25 17:05
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

"""

https://www.cnblogs.com/jfdwd/p/11150879.html
https://cloud.tencent.com/developer/article/1471179

https://blog.csdn.net/lanchunhui/article/details/79810550
"""
import pandas as pd
import featuretools as ft

df = pd.DataFrame({'ID': list(range(10))})
print(df.head())
es = ft.EntitySet(id='clients')

# 表必须要有索引, 默认第一列（无索引造索引）
es.entity_from_dataframe('df_test',
                         df,
                         index='ID', make_index=False,  # 指定生成新的索引列
                         variable_types=None,

                         time_index=None,
                         secondary_time_index=None,
                         already_sorted=False)
es.normalize_entity # 拆表
print(es['df_test'].df)

# Primitive：agg_primitives: 20 个, trans_primitives: 55 个, where_primitives
# 重写：https://docs.featuretools.com/automated_feature_engineering/primitives.html#defining-custom-primitives
from featuretools.primitives import make_agg_primitive, make_trans_primitive
primitives = ft.list_primitives()
pd.options.display.max_colwidth = 100
primitives[primitives['type'] == 'aggregation'].head(10)

from featuretools.selection import remove_low_information_features









#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'Optimizer'
__author__ = 'JieYuan'
__mtime__ = '19-3-18'
"""
from abc import abstractmethod
from bayes_opt import BayesianOptimization
from lightgbm import LGBMClassifier
from sklearn.model_selection import cross_val_score


class Optimizer(object):

    def __init__(self, X, y):
        """https://www.cnblogs.com/wzdLY/p/9867719.html
        """
        self.X = X
        self.y = y

    def maximize(self, n_iter=3, seed=2019, return_best_params=True):
        self.optimizer = BayesianOptimization(
            f=self.objective,
            pbounds=self.pbounds,
            random_state=seed,
            verbose=2
        )
        self.optimizer.maximize(init_points=5, n_iter=n_iter)

        if return_best_params:
            return self.optimizer.max  # TODO 把固定参数更新进来

    @abstractmethod
    def objective(self, **params):
        """重写目标函数"""
        self.params = params

        # 超参
        params['num_leaves'] = int(params['num_leaves'])
        params['min_child_samples'] = int(params['min_child_samples'])

        # 固定参数：TODO 更方便的方式
        params['n_estimators'] = 3000
        params['subsample_freq'] = 6  # 需要调参不
        params['verbosity'] = -1
        params['n_jobs'] = 16

        # self.params = params
        self.clf = LGBMClassifier(**params)
        _ = cross_val_score(self.clf, self.X, self.y, scoring='roc_auc', cv=5, n_jobs=5).mean()
        return _

    @abstractmethod
    @staticmethod
    def pbounds():
        """通过左边界判断参数数据类型"""
        _ = {
            'num_leaves': (2 ** 2, 2 ** 8),
            'subsample_freq': (2, 6),
            'min_child_samples': (16, 128),

            'learning_rate': (0.001, 0.1),
            'min_split_gain': (0.001, 1),
            'min_child_weight': (0.001, 100),
            'subsample': (0.3, 1),
            'colsample_bytree': (0.3, 1),
            'reg_alpha': (0.01, 10),
            'reg_lambda': (0.01, 10)
        }
        return _



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'matplotlib_chinese'
__author__ = 'JieYuan'
__mtime__ = '19-4-1'
"""
import os
import matplotlib
import matplotlib.pyplot as plt
from pathlib import Path

from matplotlib.font_manager import _rebuild


def chinese_setting(url="http://cnbj1.fds.api.xiaomi.com/browser-algo-nanjing/data/SimHei.ttf"):
    """
    :param url: SimHei字体下载链接
    :return:
    """
    print('开始设置中文...')
    matplotlibrc_path = Path(matplotlib.matplotlib_fname())
    ttf_path = matplotlibrc_path.parent.__str__() + '/fonts/ttf'
    ttf_url = 'https://raw.githubusercontent.com/Jie-Yuan/Jie-Yuan.github.io/master/SimHei.ttf' if url is None else url
    if list(Path(ttf_path).glob('SimHei.ttf')):
        pass
    else:
        print('下载字体...')
        os.popen("cd %s && wget %s" % (ttf_path, ttf_url)).read()
        os.popen("rm -rf ~/.matplotlib/*.cache").read()

    print('设置字体...')
    setting1 = 'font.family: sans-serif'
    setting2 = 'font.sans-serif: SimHei, Bitstream Vera Sans, Lucida Grande, Verdana, Geneva, Lucid, Arial, Helvetica, Avant Garde, sans-serif'
    setting3 = 'axes.unicode_minus: False'
    os.system('echo > %s' % matplotlibrc_path)
    os.system('echo %s >> %s' % (setting1, matplotlibrc_path))
    os.system('echo %s >> %s' % (setting2, matplotlibrc_path))
    os.system('echo %s >> %s' % (setting3, matplotlibrc_path))

    _rebuild()
    print('请重启kernel测试...')


if __name__ == '__main__':
    chinese_setting()
    fig, ax = plt.subplots()
    ax.plot([1, 2, 3], label='中文测试')
    ax.legend()
    plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'translate'
__author__ = 'JieYuan'
__mtime__ = '19-3-1'
"""
import requests
import random
import hashlib

"""可以通过docker增加并发
https://www.cnblogs.com/fanyang1/p/9414088.html
https://github.com/ssut/py-googletrans

https://github.com/openlabs/Microsoft-Translator-Python-API
https://github.com/cognitect/transit-python
"""
from .tencent import trans_tencent
from googletrans import Translator
from .youdao import trans_youdao

translator = Translator(service_urls=['translate.google.cn', 'translate.google.com'],
                        timeout=3)


def trans_google(q='苹果', fromLang='auto', toLang='en'):
    """

    :param q:
    :param fromLang:
    :param toLang: zh
    :return:
    """
    url = "http://translate.google.cn/translate_a/single?client=gtx&dt=t&dj=1&ie=UTF-8&sl=%s&tl=%s" % (fromLang, toLang)
    try:
        r = requests.get(url, {'q': q}, timeout=3)
        text = r.json()['sentences'][0]['trans']
    except Exception as e:
        print(e)
        text = translator.translate(q, toLang, fromLang).text
    return text


def trans_baidu(q='苹果', fromLang='auto', toLang='en'):
    """

    :param q:
    :param fromLang:
    :param toLang: zh简体 en英文
    :return:
    """
    url = "http://api.fanyi.baidu.com/api/trans/vip/translate"
    appid = '20190718000319131'  # 你的appid
    secretKey = 'goP6CsXs6sVamHtRGdBa'  # 你的密钥
    salt = str(random.randint(32768, 65536))
    # 生成签名
    sign = appid + q + salt + secretKey
    sign = hashlib.md5(sign.encode()).hexdigest()
    data = {
        "appid": appid,
        "q": q,
        "from": fromLang,
        "to": toLang,
        "salt": salt,
        "sign": sign,
    }
    r = requests.post(url, data=data, timeout=3)
    return r.json().get('trans_result')[0].get("dst")


if __name__ == '__main__':
    print(trans_tencent())
    print(trans_google('apple', 'en', 'zh'))
    print(trans_baidu('apple', 'en', 'zh'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-1-2'
"""

from .baseline_lgb import BaselineLGB
from .baseline_xgb import BaselineXGB
from .bayes_opt_lgb import BayesOptLGB
from .bayes_opt_xgb import BayesOptXGB

# _clf = LGBMClassifier(n_estimators=1)
# X = iris.data[:100, :]
# y = iris.target[:100]
# _clf.fit(X, y)
# show_info = ['split_gain', 'internal_value', 'internal_count', 'leaf_count']
# lgb.plot_tree(_clf.booster_, figsize=(60, 80), show_info=show_info)
#
# model = _clf.booster_.dump_model()
# tree_infos = model['tree_info'] # xgb_._Booster.get_dump()


# #!/usr/bin/env python
# # -*- coding: utf-8 -*-
# """
# __title__ = '__init__.py'
# __author__ = 'JieYuan'
# __mtime__ = '19-1-11'
# """
#
# ## 将xgb和lgb结果进行stacking
# train_stack = np.vstack([oof_lgb, oof_xgb]).transpose()
# test_stack = np.vstack([predictions_lgb, predictions_xgb]).transpose()
#
# folds_stack = RepeatedKFold(n_splits=5, n_repeats=2, random_state=2019)
# oof_stack = np.zeros(train_stack.shape[0])
# predictions = np.zeros(test_stack.shape[0])
#
# for fold_, (trn_idx, val_idx) in enumerate(folds_stack.split(train_stack, target)):
#     print("fold {}".format(fold_))
#     trn_data, trn_y = train_stack[trn_idx], target.iloc[trn_idx].values
#     val_data, val_y = train_stack[val_idx], target.iloc[val_idx].values
#
#     clf_3 = BayesianRidge()
#     clf_3.fit(trn_data, trn_y)
#
#     oof_stack[val_idx] = clf_3.predict(val_data)
#     predictions += clf_3.predict(test_stack) / 10
#
# print("CV score: {:<8.8f}".format(mean_absolute_error(oof_stack, target)))
# print('final score: {:<8.8f}'.format(1 / (1 + mean_absolute_error(oof_stack, target))))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'check_coverage'
__author__ = 'JieYuan'
__mtime__ = '19-1-31'
"""

# from gensim.models import KeyedVectors
#
# news_path = '../input/embeddings/GoogleNews-vectors-negative300/GoogleNews-vectors-negative300.bin'
# embeddings_index = KeyedVectors.load_word2vec_format(news_path, binary=True)
#
# import operator
#
#
# def check_coverage(vocab, embeddings_index):
#     a = {}
#     oov = {}
#     k = 0
#     i = 0
#     for word in tqdm(vocab):
#         try:
#             a[word] = embeddings_index[word]
#             k += vocab[word]
#         except:
#
#             oov[word] = vocab[word]
#             i += vocab[word]
#             pass
#
#     print('Found embeddings for {:.2%} of vocab'.format(len(a) / len(vocab)))
#     print('Found embeddings for  {:.2%} of all text'.format(k / (k + i)))
#     sorted_x = sorted(oov.items(), key=operator.itemgetter(1))[::-1]
#
#     return sorted_x
#
#
# if __name__ == '__main__':
#     oov = check_coverage(vocab, embeddings_index)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'get_label'
__author__ = 'JieYuan'
__mtime__ = '19-1-28'
"""
import numpy as np
import pandas as pd


def get_threshold_or_label(preds, psr, only_return_threshold=False):
    """默认: 指标>阈值为正样本
    :param psr: Positive sample ratio
    :return:
    """
    threshold = pd.Series(preds).quantile(1 - psr)
    if only_return_threshold:
        return threshold
    else:
        return np.where(preds > threshold, 1, 0)

# def threshold_search(y_true, y_proba):
#     best_threshold = 0
#     best_score = 0
#     for threshold in tqdm([i * 0.01 for i in range(100)]):
#         score = f1_score(y_true=y_true, y_pred=y_proba > threshold)
#         if score > best_score:
#             best_threshold = threshold
#             best_score = score
#     search_result = {'threshold': best_threshold, 'f1': best_score}
#     return search_result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'read2write'
__author__ = 'JieYuan'
__mtime__ = '19-3-15'
"""

from tqdm import tqdm


def read2write(input_file, output_file, progress_func=lambda x: x, lower=True, overwrite=True):
    with open(input_file, 'r', encoding='utf8') as input, \
            open(output_file, 'w' if overwrite else 'a', encoding='utf8') as output:
        for line in tqdm(input):
            if line:
                _ = progress_func(line.lower() if lower else line).strip()
                if _:
                    output.write(_ + '\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : OOFlgb
# @Time         : 2019-06-23 23:41
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import time
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.model_selection import StratifiedKFold, RepeatedStratifiedKFold, KFold, cross_val_predict, cross_validate
from sklearn.metrics import roc_auc_score

from lightgbm import LGBMClassifier

import os
import socket

if socket.gethostname() == 'yuanjie-Mac.local':
    os.environ['KMP_DUPLICATE_LIB_OK'] = 'TRUE'


class LGBMClassifierCV(object):
    """cross_val_predict"""

    def __init__(self, params=None, cv=5, random_state=None, n_repeats=None):
        self.clf = LGBMClassifier()
        if params:
            self.clf.set_params(**params)
        if n_repeats:
            self._kf = RepeatedStratifiedKFold(cv, True, random_state)
            self._num_preds = cv * n_repeats
        else:
            self._kf = StratifiedKFold(cv, True, random_state)
            self._num_preds = cv

    def fit(self, X, y, X_test, feval=roc_auc_score, sample_weight=None, init_score=None, eval_metric='auc',
            early_stopping_rounds=100, verbose=100, feature_name='auto', categorical_feature='auto', callbacks=None):
        """输入数组"""

        self.oof_train = np.zeros(len(X))
        self.oof_test = np.zeros((len(X_test), self._num_preds))
        for n_fold, (train_index, valid_index) in enumerate(self._kf.split(X, y)):
            if verbose:
                print("\033[94mFold %s started at %s\033[0m" % (n_fold + 1, time.ctime()))
            X_train, y_train = X[train_index], y[train_index]
            X_valid, y_valid = X[valid_index], y[valid_index]
            eval_set = [(X_train, y_train), (X_valid, y_valid)]

            ########################################################################
            self.clf.fit(X_train, y_train, sample_weight, init_score, eval_set, eval_names=('Train', 'Valid'),
                         eval_sample_weight=None, eval_class_weight=None, eval_init_score=None, eval_metric=eval_metric,
                         early_stopping_rounds=early_stopping_rounds, verbose=verbose, feature_name=feature_name,
                         categorical_feature=categorical_feature, callbacks=callbacks)

            self.oof_train[valid_index] = self.clf.predict_proba(X_valid)[:, 1]
            self.oof_test[:, n_fold] = self.clf.predict_proba(X_test)[:, 1]
            ########################################################################

        # 输出 测试集 oof
        self.oof_test_rank = (pd.DataFrame(self.oof_test).rank().mean(1) / len(self.oof_test)).values
        self.oof_test = self.oof_test.mean(1)

        # 计算 训练集 oof 得分
        if feval:
            score = feval(y, self.oof_train)
            print(f"\n\033[94mCV Sorce: {score} ended at {time.ctime()}\033[0m")
            return score

    def oof_submit(self, ids, pred_ranking=False, file='submit.csv'):
        print(f'Save {file} ...')
        preds = self.oof_test if pred_ranking else self.oof_test_rank
        if not isinstance(ids, pd.DataFrame):
            ids = pd.DataFrame(ids)
        ids.assign(preds=preds).to_csv(file, index=False, header=False)

    def oof_save(self, file='./oof_train_and_test.csv'):
        assert isinstance(file, str)
        _ = np.append(self.oof_train, self.oof_test)
        pd.DataFrame(_, columns=['oof_train_and_test']).to_csv(file, index=False)

    def plot_feature_importances(self, feature_names, topk=20, figsize=(10, 6)):
        columns = ['Importances', 'Features']
        importances = self.clf.feature_importances_
        df = pd.DataFrame(list(zip(importances, feature_names)), columns=columns).sort_values('Importances', 0, False)

        plt.figure(figsize=figsize)
        sns.barplot(*columns, data=df[:topk])
        plt.title('Features Importances\n')
        plt.tight_layout()
        plt.savefig('lgbm_importances.png')


if __name__ == "__main__":
    from sklearn.datasets import make_classification

    X, y = make_classification()
    X_test, _ = make_classification()

    clf = LGBMClassifierCV()
    clf.fit(X, y, X_test)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : sk_cv
# @Time         : 2019-09-16 11:32
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from lightgbm import LGBMClassifier
from sklearn.datasets import make_classification
from sklearn.model_selection import cross_val_score

X, y = make_classification(
    n_samples=1000,
    n_features=45,
    n_informative=12,
    n_redundant=7,
    random_state=134985745,
)

from tql.algo_ml.opt_params.Optimizer import Optimizer


class Opt(Optimizer):

    def objective(self, **params):
        params['n_jobs'] = 16
        params['subsample_freq'] = 6
        params['verbosity'] = -1
        params['num_leaves'] = int(params['num_leaves'])
        params['min_child_samples'] = int(params['min_child_samples'])

        self.clf = LGBMClassifier(**params)
        _ = cross_val_score(self.clf, self.X, self.y, scoring='roc_auc', cv=5, n_jobs=5).mean()
        return _



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : train_lgb
# @Time         : 2019-07-30 16:08
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from tql.spark import SparkInit
import pyspark
from pyspark.sql.types import *

import pyspark.sql.functions as F
from pyspark.ml.feature import Word2Vec

spark = pyspark.sql.SparkSession.builder.appName("MyApp") \
    .config("spark.jars.packages", "Azure:mmlspark:0.17") \
    .getOrCreate()

from mmlspark import LightGBMClassifier
from pyspark.ml.classification import LogisticRegression
from sklearn.datasets import make_classification
import pandas as pd

X, y = make_classification()
df_pandas = pd.DataFrame(X).assign(label=y)
df_pandas.columns = [str(i) for i in df_pandas.columns]
df = spark.createDataFrame(df_pandas)

model = mmlspark.TrainClassifier(model=LightGBMClassifier(), labelCol='label').fit(df)
prediction = model.transform(df)
prediction.show()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'AggFeat'
__author__ = 'JieYuan'
__mtime__ = '19-1-15'
"""
from .Funcs import Funcs
from tqdm import tqdm
from concurrent.futures import ProcessPoolExecutor

import pandas as pd


class GroupFeats(object):

    def __init__(self, df, cat_cols, num_cols):
        self.df = df.copy()
        self.cat_cols = cat_cols
        self.num_cols = num_cols
        self.num_funcs = ['min', 'mean', 'median', 'max', 'sum', 'std', 'var', 'sem',
                          'skew'] + Funcs().num_funcs
        self.cat_funcs = ['nunique', 'max', 'min'] + Funcs().cat_funcs

    def transform(self, max_workers=4):
        with ProcessPoolExecutor(min(max_workers, len(self.cat_cols))) as pool:
            for _df in pool.map(self._get_agg_feats, tqdm(self.cat_cols, 'agg ...')):
                self.df = pd.merge(self.df, _df, 'left')
            return self.df

    def _get_agg_feats(self, key_cols):
        if isinstance(key_cols, str):
            key_cols = [key_cols]
        num_feats = self.num_cols
        cat_feats = list(set(self.cat_cols) - set(key_cols))

        gr = self.df.groupby(key_cols)
        trans_dict = dict(zip(num_feats + cat_feats + key_cols,
                              [self.num_funcs] * len(num_feats) \
                              + [self.cat_funcs] * len(cat_feats) \
                              + [['count']] * len(key_cols)))
        df = gr.agg(trans_dict)
        df.columns = ['&'.join(key_cols) + '_' + '_'.join(i) for i in df.columns]
        return df.reset_index()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : TextRNNBiAtt
# @Time         : 2019-06-24 11:36
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from .BaseModel import BaseModel
from tensorflow.python.keras import Input, Model
from tensorflow.python.keras.layers import Dense
from tensorflow.python.keras.layers import Attention, AdditiveAttention
from tensorflow.python.keras.layers import Bidirectional, LSTM, CuDNNLSTM, GRU, CuDNNGRU


class TextRNNBiAtt(BaseModel):
    """
    TODO:
    """

    def __init__(self, max_tokens, maxlen=128, embedding_size=None, num_class=1, weights=None, rnn=GRU):
        """
        :param rnn: LSTM, CuDNNLSTM, GRU, CuDNNGRU
        """
        super().__init__(max_tokens, maxlen, embedding_size, num_class, weights)
        self._RNN = rnn

    def get_model(self):
        input = Input((self.maxlen,))

        embedding = self.embedding_layer(input)
        x = Bidirectional(self._RNN(128, return_sequences=True))(embedding)  # LSTM or GRU
        # TODO: keras 官方版 attention 怎么用
        x = Attention(self.maxlen)(x)

        output = Dense(self.num_class, activation=self.last_activation)(x)
        model = Model(inputs=input, outputs=output)
        return model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'tencent_word_vectors'
__author__ = 'JieYuan'
__mtime__ = '19-2-13'
"""
import numpy as np
from tqdm import tqdm

fname = '/home/yuanjie/desktop/data/Tencent_AILab_ChineseEmbedding/Tencent_AILab_ChineseEmbedding.txt'
idf = {}
with open(fname) as f:
    for line in tqdm(f, 'Load Vectors ...'):
        line = line.strip().split()
        if len(line) > 2:
            idf[line[0]] = np.linalg.norm(np.asarray(line[-200:], dtype='float32'), 2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'Doc2Vec'
__author__ = 'JieYuan'
__mtime__ = '2019-05-23'
"""
from tqdm import tqdm
from pathlib import Path
from gensim.models.doc2vec import Doc2Vec, TaggedDocument, TaggedLineDocument


class GensimDoc2Vec(object):
    def __init__(self):
        pass

    def fit(self, corpus, vector_size=300, window=10, min_count=1, dm=1, hs=0, negative=5,
            epochs=10, workers=8):

        if isinstance(corpus, str) and Path(corpus).is_file():
            corpus = TaggedLineDocument(corpus)
        else:
            corpus = [TaggedDocument(line, [idx]) for idx, line in enumerate(corpus)]

        model = Doc2Vec(documents=tqdm(corpus), vector_size=vector_size,
                        window=window, min_count=min_count, dm=dm,
                        hs=hs, negative=negative, epochs=epochs, workers=workers)
        return model


if __name__ == '__main__':
    docs = [['Well', 'done!'],
            ['Good', 'work'],
            ['Great', 'effort'],
            ['nice', 'work'],
            ['Excellent!'],
            ['Weak'],
            ['Poor', 'effort!'],
            ['not', 'good'],
            ['poor', 'work'],
            ['Could', 'have', 'done', 'better.']]
    model = GensimDoc2Vec()
    model.fit(docs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'daily'
__author__ = 'JieYuan'
__mtime__ = '2019-05-24'
"""

# 1. 每5分钟爬一次
# 2. 每天存文件
import time
import pendulum
import pandas as pd
from iwork.crawler.HotSpider import HotSpider

get_time = lambda t: pendulum.from_timestamp(t, 'Asia/Shanghai')

query = '正能量'
hs = HotSpider(query)

data = pd.DataFrame()
while 1:
    try:
        df = hs.df_sites_info()
        dt = get_time(time.time())

        print("我睡一会😪💤")
        time.sleep(10)

        if dt.add(minutes=5).date() < dt.date():
            continue
        else:
            print("我取")
            data = pd.concat([df, data]).drop_duplicates('title')
            data.to_csv("positive energy: %s" % dt.date(), '\t', index=False)

    except Exception as e:
        print(e)
        raise Exception("爬取异常")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'feval'
__author__ = 'JieYuan'
__mtime__ = '19-3-18'
"""
import wrapt

def feval(multiclass=None, is_bigger_better=True, model='lgb'):
    """example
    @feval(3)
    def f1_score(y_pred, y_true):
        '注意入参顺序'
        return f1_score(y_true, y_pred, average='macro')
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        y_pred, y_true = args
        y_true = y_true.get_label()
        if model == 'lgb':
            if multiclass:
                y_pred = np.array(y_pred).reshape(multiclass, -1).argmax(0)
            return wrapped.__name__, wrapped(y_pred, y_true), is_bigger_better
        elif is_bigger_better:
            """xgb评估指标默认越小越好"""
            return '-' + wrapped.__name__, - wrapped(y_pred, y_true)
        else:
            return wrapped.__name__, wrapped(y_pred, y_true)

    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : XGBOptimizer
# @Time         : 2019-09-16 11:58
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from .Optimizer import Optimizer


class XGBOptimizer(Optimizer):


    def pbounds():
        pass

    def objective(self, **params):
        pass

#
# dict(
#             max_depth=7,
#             learning_rate=(),
#
#             gamma=0.0,  # 描述分裂的最小 gain, 控制树的有用的分裂
#             min_child_weight=1,  # 决定最小叶子节点样本权重和,使一个结点分裂的最小权值之和, 避免过拟合
#
#             subsample=0.8,
#             colsample_bytree=0.8,  # 每棵树的列数
#             colsample_bylevel=0.8,  # 每一层的列数
#
#             reg_alpha=0.0,
#             reg_lambda=0.0,
#
#             scale_pos_weight=scale_pos_weight,
#
#             random_state=seed,
#             n_jobs=n_jobs,
#             silent=True
#         )


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'Ngrams'
__author__ = 'JieYuan'
__mtime__ = '2019/4/16'
"""
from itertools import chain, combinations


##########################################################################
# Ngram iteration
##########################################################################
def pad_sequence(
        sequence,
        n,
        pad_left=False,
        pad_right=False,
        left_pad_symbol=None,
        right_pad_symbol=None,
):
    """
    Returns a padded sequence of items before ngram extraction.

        >>> list(pad_sequence([1,2,3,4,5], 2, pad_left=True, pad_right=True, left_pad_symbol='<s>', right_pad_symbol='</s>'))
        ['<s>', 1, 2, 3, 4, 5, '</s>']
        >>> list(pad_sequence([1,2,3,4,5], 2, pad_left=True, left_pad_symbol='<s>'))
        ['<s>', 1, 2, 3, 4, 5]
        >>> list(pad_sequence([1,2,3,4,5], 2, pad_right=True, right_pad_symbol='</s>'))
        [1, 2, 3, 4, 5, '</s>']

    :param sequence: the source data to be padded
    :type sequence: sequence or iter
    :param n: the degree of the ngrams
    :type n: int
    :param pad_left: whether the ngrams should be left-padded
    :type pad_left: bool
    :param pad_right: whether the ngrams should be right-padded
    :type pad_right: bool
    :param left_pad_symbol: the symbol to use for left padding (default is None)
    :type left_pad_symbol: any
    :param right_pad_symbol: the symbol to use for right padding (default is None)
    :type right_pad_symbol: any
    :rtype: sequence or iter
    """
    sequence = iter(sequence)
    if pad_left:
        sequence = chain((left_pad_symbol,) * (n - 1), sequence)
    if pad_right:
        sequence = chain(sequence, (right_pad_symbol,) * (n - 1))
    return sequence


# add a flag to pad the sequence so we get peripheral ngrams?


def ngrams(
        sequence,
        n,
        pad_left=False,
        pad_right=False,
        left_pad_symbol=None,
        right_pad_symbol=None,
):
    """
    Return the ngrams generated from a sequence of items, as an iterator.
    For example:

        >>> from nltk.util import ngrams
        >>> list(ngrams([1,2,3,4,5], 3))
        [(1, 2, 3), (2, 3, 4), (3, 4, 5)]

    Wrap with list for a list version of this function.  Set pad_left
    or pad_right to true in order to get additional ngrams:

        >>> list(ngrams([1,2,3,4,5], 2, pad_right=True))
        [(1, 2), (2, 3), (3, 4), (4, 5), (5, None)]
        >>> list(ngrams([1,2,3,4,5], 2, pad_right=True, right_pad_symbol='</s>'))
        [(1, 2), (2, 3), (3, 4), (4, 5), (5, '</s>')]
        >>> list(ngrams([1,2,3,4,5], 2, pad_left=True, left_pad_symbol='<s>'))
        [('<s>', 1), (1, 2), (2, 3), (3, 4), (4, 5)]
        >>> list(ngrams([1,2,3,4,5], 2, pad_left=True, pad_right=True, left_pad_symbol='<s>', right_pad_symbol='</s>'))
        [('<s>', 1), (1, 2), (2, 3), (3, 4), (4, 5), (5, '</s>')]


    :param sequence: the source data to be converted into ngrams
    :type sequence: sequence or iter
    :param n: the degree of the ngrams
    :type n: int
    :param pad_left: whether the ngrams should be left-padded
    :type pad_left: bool
    :param pad_right: whether the ngrams should be right-padded
    :type pad_right: bool
    :param left_pad_symbol: the symbol to use for left padding (default is None)
    :type left_pad_symbol: any
    :param right_pad_symbol: the symbol to use for right padding (default is None)
    :type right_pad_symbol: any
    :rtype: sequence or iter
    """
    sequence = pad_sequence(
        sequence, n, pad_left, pad_right, left_pad_symbol, right_pad_symbol
    )

    history = []
    while n > 1:
        # PEP 479, prevent RuntimeError from being raised when StopIteration bubbles out of generator
        try:
            next_item = next(sequence)
        except StopIteration:
            # no more data, terminate the generator
            return
        history.append(next_item)
        n -= 1
    for item in sequence:
        history.append(item)
        yield tuple(history)
        del history[0]


def bigrams(sequence, **kwargs):
    """
    Return the bigrams generated from a sequence of items, as an iterator.
    For example:

        >>> from nltk.util import bigrams
        >>> list(bigrams([1,2,3,4,5]))
        [(1, 2), (2, 3), (3, 4), (4, 5)]

    Use bigrams for a list version of this function.

    :param sequence: the source data to be converted into bigrams
    :type sequence: sequence or iter
    :rtype: iter(tuple)
    """

    for item in ngrams(sequence, 2, **kwargs):
        yield item


def trigrams(sequence, **kwargs):
    """
    Return the trigrams generated from a sequence of items, as an iterator.
    For example:

        >>> from nltk.util import trigrams
        >>> list(trigrams([1,2,3,4,5]))
        [(1, 2, 3), (2, 3, 4), (3, 4, 5)]

    Use trigrams for a list version of this function.

    :param sequence: the source data to be converted into trigrams
    :type sequence: sequence or iter
    :rtype: iter(tuple)
    """

    for item in ngrams(sequence, 3, **kwargs):
        yield item


def everygrams(sequence, min_len=1, max_len=-1, **kwargs):
    """
    Returns all possible ngrams generated from a sequence of items, as an iterator.

        >>> sent = 'a b c'.split()
        >>> list(everygrams(sent))
        [('a',), ('b',), ('c',), ('a', 'b'), ('b', 'c'), ('a', 'b', 'c')]
        >>> list(everygrams(sent, max_len=2))
        [('a',), ('b',), ('c',), ('a', 'b'), ('b', 'c')]

    :param sequence: the source data to be converted into trigrams
    :type sequence: sequence or iter
    :param min_len: minimum length of the ngrams, aka. n-gram order/degree of ngram
    :type  min_len: int
    :param max_len: maximum length of the ngrams (set to length of sequence by default)
    :type  max_len: int
    :rtype: iter(tuple)
    """

    if max_len == -1:
        max_len = len(sequence)
    for n in range(min_len, max_len + 1):
        for ng in ngrams(sequence, n, **kwargs):
            yield ng


def skipgrams(sequence, n, k, **kwargs):
    """
    Returns all possible skipgrams generated from a sequence of items, as an iterator.
    Skipgrams are ngrams that allows tokens to be skipped.
    Refer to http://homepages.inf.ed.ac.uk/ballison/pdf/lrec_skipgrams.pdf

        >>> sent = "Insurgents killed in ongoing fighting".split()
        >>> list(skipgrams(sent, 2, 2))
        [('Insurgents', 'killed'), ('Insurgents', 'in'), ('Insurgents', 'ongoing'), ('killed', 'in'), ('killed', 'ongoing'), ('killed', 'fighting'), ('in', 'ongoing'), ('in', 'fighting'), ('ongoing', 'fighting')]
        >>> list(skipgrams(sent, 3, 2))
        [('Insurgents', 'killed', 'in'), ('Insurgents', 'killed', 'ongoing'), ('Insurgents', 'killed', 'fighting'), ('Insurgents', 'in', 'ongoing'), ('Insurgents', 'in', 'fighting'), ('Insurgents', 'ongoing', 'fighting'), ('killed', 'in', 'ongoing'), ('killed', 'in', 'fighting'), ('killed', 'ongoing', 'fighting'), ('in', 'ongoing', 'fighting')]

    :param sequence: the source data to be converted into trigrams
    :type sequence: sequence or iter
    :param n: the degree of the ngrams
    :type n: int
    :param k: the skip distance
    :type  k: int
    :rtype: iter(tuple)
    """

    # Pads the sequence as desired by **kwargs.
    if 'pad_left' in kwargs or 'pad_right' in kwargs:
        sequence = pad_sequence(sequence, n, **kwargs)

    # Note when iterating through the ngrams, the pad_right here is not
    # the **kwargs padding, it's for the algorithm to detect the SENTINEL
    # object on the right pad to stop inner loop.
    SENTINEL = object()
    for ngram in ngrams(sequence, n + k, pad_right=True, right_pad_symbol=SENTINEL):
        head = ngram[:1]
        tail = ngram[1:]
        for skip_tail in combinations(tail, n - 1):
            if skip_tail[-1] is SENTINEL:
                continue
            yield head + skip_tail



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'semi_simple'
__author__ = 'JieYuan'
__mtime__ = '19-1-11'
"""
import numpy as np
import pandas as pd
from ....pipe import tqdm, cprint


class SimpleSemi(object):

    def __init__(self, clf, subsample=0.05, n_iter=1, scale_pos=1, mode=None, opt_seed=None):
        """

        :param subsample:
        :param n_iter:
        :param scale_pos: 正样本 / 负样本
        :param mode:
            'p': 从X_test, 只采样正样本
            'n': 从X_test, 只采样正样本
            None: 从X_test, 采样正+负样本
        """
        self.clf = clf
        self.subsample = subsample
        self.n_iter = n_iter
        self.scale_pos = scale_pos
        self.mode = mode
        self.opt_seed = opt_seed

        _ = "X_train will stack ≈ {:.2f}% of X_test".format(
            (1 - (1 - subsample - subsample * scale_pos) ** n_iter) * 100)
        cprint(_)

    def fit(self, X, y, X_test):
        self.X = X
        self.y = y
        self.X_test = np.asarray(X_test)

        for _ in tqdm(range(self.n_iter + 1)):
            self.clf.fit(self.X, self.y)
            _ = pd.Series(self.clf.predict_proba(self.X_test)[:, 1])

            if self.mode == 'n':
                pred = _.mask(lambda x: x < x.quantile(self.subsample), 0)
            elif self.mode == 'p':
                pred = _.mask(lambda x: x > x.quantile(1 - self.subsample * self.scale_pos), 1)
            else:
                pred = (_.mask(lambda x: x < x.quantile(self.subsample), 0)
                        .mask(lambda x: x > x.quantile(1 - self.subsample * self.scale_pos), 1))

            pred_ = pred[lambda x: x.isin([0, 1])]
            pseudo_label_idx = pred_.index
            no_pseudo_label_idx = pred.index.difference(pseudo_label_idx)

            self.X = np.row_stack((self.X, self.X_test[pseudo_label_idx]))
            self.y = np.hstack((self.y, pred_))

            self.X_test = self.X_test[no_pseudo_label_idx]
        return self.clf



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : youdao
# @Time         : 2019-08-07 13:21
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
import time

import requests
import hashlib


def trans_youdao(q="苹果", fromLang='auto', toLang='en'):
    t = str(int(time.time()))
    c = "rY0D^0'nM0}g5Mm1z%1G4"
    u = 'fanyideskweb'
    creatmd5 = u + q + t + c

    # 生成md5
    md5 = hashlib.md5()
    md5.update(creatmd5.encode('utf-8'))
    sign = md5.hexdigest()

    data = {}
    data['i'] = q
    data['from'] = fromLang
    data['to'] = toLang
    data['smartresult'] = 'dict'
    data['client'] = 'fanyideskweb'
    data['salt'] = t
    data['sign'] = sign
    data['doctype'] = 'json'
    data['version'] = '2.1'
    data['keyfrom'] = 'fanyi.web'
    data['action'] = 'FY_BY_CLICKBUTTION'
    data['typoResult'] = 'true'

    url = 'http://fanyi.youdao.com/translate?smartresult=dict&smartresult=rule&sessionFrom='
    return requests.post(url, data).json()['translateResult']


if __name__ == '__main__':
    print(trans_tencent('apple', toLang="zh"))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : tfidf_lr
# @Time         : 2019-06-20 12:02
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
from sklearn.pipeline import make_pipeline
from sklearn.linear_model import LogisticRegression
from sklearn.feature_extraction.text import TfidfVectorizer


class BaselineBow(object):
    """
    from sklearn.naive_bayes import MultinomialNB
    from sklearn.linear_model import SGDClassifier
    from sklearn.linear_model import LogisticRegression
    mnb = MultinomialNB()
    svm = SGDClassifier(loss='hinge', n_iter=100)
    lr = LogisticRegression()
    """

    def __init__(self, estimator=LogisticRegression(), tokenizer=None, vectorizer=None):
        self._estimator = estimator
        self._vectorizer = vectorizer if vectorizer else TfidfVectorizer()
        self._vectorizer.tokenizer = tokenizer

    def __call__(self, *args, **kwargs):
        return self._pipline

    @property
    def _pipline(self):
        return make_pipeline(self._vectorizer, self._estimator)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : demo
# @Time         : 2019-08-09 13:16
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import PySimpleGUI as sg

# layout = [
#     [sg.Text('Enter a Number')],
#     [sg.Input()],
#     [sg.OK()]
# ]
#
# event, (number,) = sg.Window('Enter a number example').Layout(layout).Read()
#
# sg.Popup(event, number)

layout = [
[sg.Text('你的学历是',auto_size_text=True)],
[sg.Checkbox('游泳',default=True)],   #h 或者 v 表示水平或者垂直
[sg.Checkbox('篮球')],
[sg.Checkbox('足球')],
[sg.Checkbox('羽毛球')],
[sg.OK('确认',auto_size_button=True)]
]

with sg.FlexForm('复选框',auto_size_text=True) as form:
    button_name,choices = form.Layout(layout).Read()
    sg.Popup(button_name,choices)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'dist'
__author__ = 'JieYuan'
__mtime__ = '2019-05-24'
"""
from scipy.spatial.distance import pdist
from functools import partial

from scipy.spatial.distance import pdist
from functools import partial


class Distance(object):

    def __init__(self, metric='cosine'):
        if metric in self.metrics:
            for metric in self.metrics:
                func = partial(pdist, metric=metric)
                setattr(self, metric, func)

    @property
    def metrics(self):
        _ = ['cosine',
             'euclidean',
             'jaccard',
             'cityblock',
             'braycurtis',
             'canberra',
             'chebyshev',
             'correlation',
             'dice',
             'hamming',
             'jensenshannon',
             'kulsinski',
             'mahalanobis',
             'matching',
             'minkowski',
             'rogerstanimoto',
             'russellrao',
             'seuclidean',
             'sokalmichener',
             'sokalsneath',
             'sqeuclidean',
             'yule']
        return _



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'rte'
__author__ = 'JieYuan'
__mtime__ = '19-1-3'
"""


class EmbeddingTrees(object):

    def embedding_random_trees(self, random_state):
        from sklearn.ensemble import RandomTreesEmbedding
        rte = RandomTreesEmbedding(
            n_estimators=256,
            max_depth=7,
            min_samples_split=2,
            min_samples_leaf=1,
            min_weight_fraction_leaf=0.0,
            max_leaf_nodes=None,
            min_impurity_decrease=0.0,
            min_impurity_split=None,
            sparse_output=True,
            n_jobs=8,
            random_state=random_state)
        return rte




#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'async'
__author__ = 'JieYuan'
__mtime__ = '2019-05-22'
"""
import time
import asyncio
from aiohttp import ClientSession

"""
TODO: 多进程+协程
"""


class Async(object):
    """https://www.cnblogs.com/shenh/p/9090586.html"""

    def __init__(self, concurrent_number=500):
        self.response = None
        self.loop = asyncio.get_event_loop()
        # self.loop.set_default_executor(ProcessPoolExecutor(10))
        self.semaphore = asyncio.Semaphore(concurrent_number)

    def gets(self, urls):
        tasks = map(lambda url: asyncio.ensure_future(self._get(url)), urls)
        _ = self.loop.run_until_complete(asyncio.gather(*tasks))  # 收集响应
        return _

    def posts(self, urls, json):
        tasks = map(lambda url: asyncio.ensure_future(self._post(url, json)), urls)
        _ = self.loop.run_until_complete(asyncio.gather(*tasks))  # 收集响应
        return _

    async def _get(self, url):
        async with self.semaphore:
            async with ClientSession() as session:
                async with session.get(url) as response:
                    print('CostTime: %s' % time.time())
                    return await response.text(encoding='utf8')

    async def _post(self, url, json):
        async with self.semaphore:
            async with ClientSession() as session:
                async with session.post(url, json=json) as response:
                    print('CostTime: %s' % time.time())
                    return await response.text(encoding='utf8')

if __name__ == '__main__':
    a = Async()

    urls = ['https://www.baidu.com/1', 'https://www.baidu.com/1'] * 100
    # urls = ["""http://dev.web.algo.browser.miui.srv/nlp/cut/jieba?text=不知道"""]*4000
    r = a.gets(urls)
    print(r)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'SimpleEDA'
__author__ = 'JieYuan'
__mtime__ = '2019-04-22'
"""
import pandas as pd

from ..utils import cprint


class SimpleEDA(object):
    """
    1. 缺失值
    2. 方差/去重类别数/确定类别特征
    """

    def __init__(self, df: pd.DataFrame, exclude=None):
        self.df = df.drop(exclude, 1, errors='ignore') if exclude else df

        # self.df.head().T.add_prefix('row_')

    def summary(self, desc_rows=10):
        self._na(desc_rows)
        self._unique(desc_rows)

    def _na(self, desc_rows=10):
        cprint("\n1. 统计缺失率...")
        self.s_na = self.df.isnull().sum()[lambda x: x > 0].sort_values(0, False) \
                    / self.df.__len__() * 100
        print(self.s_na.head(desc_rows))

    def _unique(self, desc_rows=10):
        cprint("\n2. 统计类别数...")
        self.s_unique = self.df.nunique(dropna=False)[lambda x: x < 1024].sort_values()
        print(self.s_unique.head(desc_rows))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : ConfigKeras
# @Time         : 2019-06-21 15:28
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import os
import random
import numpy as np
import tensorflow as tf
from tensorflow.python.keras import backend as K


class ConfigKeras(object):
    """
    https://www.cnblogs.com/wuliytTaotao/p/10883749.html
    """

    def __init__(self, seed=2019):
        self._seed = seed

    def set_seed(self):
        os.environ["PYTHONHASHSEED"] = '0'
        os.environ["CUDA_VISIBLE_DEVICES"] = "-1"

        random.seed(self._seed)
        np.random.seed(self._seed)
        # config = tf.estimator.RunConfig(tf_random_seed=234)
        # tf.contrib.learn.RunConfig(tf_random_seed=234)

        tf.set_random_seed(self._seed)
        session_conf = tf.ConfigProto(intra_op_parallelism_threads=1,
                                      inter_op_parallelism_threads=1)
        sess = tf.Session(graph=tf.get_default_graph(), config=session_conf)  # K.get_session().graph.as_default()
        K.set_session(sess)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : subsets
# @Time         : 2019-07-25 22:40
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


def PowerSetsBinary(items, min_len=2, max_len=None):
    N = len(items)
    if max_len is None:
        max_len = N
    else:
        max_len = min(max_len, N)
    for i in range(2 ** N):  # 子集的个数
        combo = []
        for j in range(N):  # 用来判断二进制数的下标为j的位置的数是否为1
            if (i >> j) % 2:
                combo.append(items[j])
        if min_len <= len(combo) <= max_len:
            yield combo



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : ExplainNLP
# @Time         : 2019-06-20 13:24
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from nlp.models import BaselineBow
from sklearn.linear_model import LogisticRegression
from lime.lime_text import LimeTextExplainer


# import eli5
# from eli5.lime import TextExplainer
#
# te = TextExplainer(random_state=42)
# te.fit(doc, pipe.predict_proba)
# te.show_prediction(target_names=twenty_train.target_names)

class ExplainerText(object):
    """
    X = df.review.astype(str).map(lambda x: ' '.join(jieba.cut(x)))
    y = df.label

    enlp = ExplainNLP()
    enlp.fit(X, y)
    enlp.explain(X[0])
    """

    def __init__(self, estimator=LogisticRegression(), class_names=None):
        self._baseline = BaselineBow(estimator)()
        self._explainer = LimeTextExplainer(verbose=True, class_names=class_names)

    def fit(self, X, y):
        self._baseline.fit(X, y)
        return self._baseline

    def explain(self, sentence, num_features=6):
        """
        :param sentence: '分词 空格 拼接'
        :param num_features:
        :return:
        """
        exp = self._explainer.explain_instance(
            sentence, self._baseline.predict_proba, num_features=num_features)

        exp.show_in_notebook(text=1 if len(sentence) < 256 else 0)

        return exp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : wandb_example
# @Time         : 2019-08-12 13:29
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from tensorflow.python.keras.datasets import fashion_mnist
from tensorflow.python.keras.models import Sequential
from tensorflow.python.keras.layers import Conv2D, MaxPooling2D, Dropout, Dense, Flatten
from tensorflow.python.keras.utils import np_utils
from tensorflow.python.keras.optimizers import SGD
from tensorflow.python.keras.callbacks import TensorBoard
import random

# Import wandb libraries
import wandb
from wandb.keras import WandbCallback

# Initialize wandb
wandb.init(project="example")
config = wandb.config


# Track hyperparameters
config.dropout = 0.2
config.hidden_layer_size = 128
config.layer_1_size = 16
config.layer_2_size = 32
config.learn_rate = 0.01
config.decay = 1e-6
config.momentum = 0.9
config.epochs = 8

(X_train_orig, y_train_orig), (X_test, y_test) = fashion_mnist.load_data()

# Reducing the dataset size to 10,000 examples for faster train time
true = list(map(lambda x: True if random.random() < 0.167 else False, range(60000)))
ind = []
for i, x in enumerate(true):
    if x == True: ind.append(i)

X_train = X_train_orig[ind, :, :]
y_train = y_train_orig[ind]

img_width = 28
img_height = 28
labels = ["T-shirt/top", "Trouser", "Pullover", "Dress",
          "Coat", "Sandal", "Shirt", "Sneaker", "Bag", "Ankle boot"]

X_train = X_train.astype('float32')
X_train /= 255
X_test = X_test.astype('float32')
X_test /= 255

# reshape input data
X_train = X_train.reshape(X_train.shape[0], img_width, img_height, 1)
X_test = X_test.reshape(X_test.shape[0], img_width, img_height, 1)

# one hot encode outputs
y_train = np_utils.to_categorical(y_train)
y_test = np_utils.to_categorical(y_test)
num_classes = y_test.shape[1]

sgd = SGD(lr=config.learn_rate, decay=config.decay, momentum=config.momentum,
          nesterov=True)

# build model
model = Sequential()
model.add(Conv2D(config.layer_1_size, (5, 5), activation='relu',
                 input_shape=(img_width, img_height, 1)))
model.add(MaxPooling2D(pool_size=(2, 2)))
model.add(Conv2D(config.layer_2_size, (5, 5), activation='relu'))
model.add(MaxPooling2D(pool_size=(2, 2)))
model.add(Dropout(config.dropout))
model.add(Flatten())
model.add(Dense(config.hidden_layer_size, activation='relu'))
model.add(Dense(num_classes, activation='softmax'))

# Add Keras WandbCallback
model.compile(loss='categorical_crossentropy', optimizer=sgd, metrics=['accuracy'])
model.fit(X_train, y_train, validation_data=(X_test, y_test), epochs=config.epochs,
          callbacks=[WandbCallback(data_type="image", labels=labels)])



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'fstring'
__author__ = 'JieYuan'
__mtime__ = '2019-05-06'
"""

'''https://blog.csdn.net/sunxb10/article/details/81036693'''
'''https://docs.python.org/3/library/string.html#format-string-syntax'''
# %[(name)][flags][width].[precision]typecode



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'FilterFeatures'
__author__ = 'JieYuan'
__mtime__ = '19-1-24'
"""
from ....utils.time import timer

import pandas as pd
from functools import partial
from tqdm import tqdm
from concurrent.futures import ProcessPoolExecutor

"""https://www.cnblogs.com/nolonely/p/6435083.html
1、为什么要做特征选择
在有限的样本数目下，用大量的特征来设计分类器计算开销太大而且分类性能差。
2、特征选择的确切含义
将高维空间的样本通过映射或者是变换的方式转换到低维空间，达到降维的目的，然后通过特征选取删选掉冗余和不相关的特征来进一步降维。
3、特征选取的原则
获取尽可能小的特征子集，不显著降低分类精度、不影响类分布以及特征子集应具有稳定适应性强等特点
"""


class FilterFeatures(object):
    """粗筛
    高缺失率：
    低方差（高度重复值）：0.5%~99.5%分位数内方差为0的初筛
    高相关：特别高的初筛，根据重要性细筛
    低重要性：
    召回高IV：
    """

    def __init__(self, df: pd.DataFrame, exclude=None):
        assert isinstance(exclude, list)
        assert isinstance(df, pd.DataFrame)

        self.df = df
        if exclude:
            self.feats = df.columns.difference(exclude)
        else:
            self.feats = df.columns.tolist()

    def run(self):
        df = self.df.copy()

        with timer('干掉高缺失'):
            to_drop = self.filter_missing()
            if to_drop:
                df.drop(to_drop, 1, inplace=True)

        with timer('干掉低方差'):
            to_drop = self.filter_variance()
            if to_drop:
                df.drop(to_drop, 1, inplace=True)

        with timer('干掉高相关'):
            pass

        return df

    def filter_missing(self, feats=None, threshold=0.95):
        """
        :param feat_cols:
        :param threshold:
        :param as_na: 比如把-99当成缺失值
        :return:
        """
        if feats is None:
            feats = self.feats

        to_drop = (self.df[feats].isna().sum() / len(self.df))[lambda x: x > threshold].index.tolist()
        print('%d features with greater than %0.2f missing values.' % (len(to_drop), threshold))
        return to_drop

    def _filter_variance(self, feat, df):
        var = df[feat][lambda x: x.between(x.quantile(0.005), x.quantile(0.995))].var()
        return '' if var else feat

    def filter_variance(self, feats=None, max_worker=4):
        if feats is None:
            feats = self.feats

        _filter_variance = partial(self._filter_variance, df=self.df)
        with ProcessPoolExecutor(min(max_worker, len(feats))) as pool:
            to_drop = pool.map(_filter_variance, tqdm(feats, 'Filter Variance ...'))
            to_drop = [feat for feat in to_drop if feat]
        print('%d features with 0 variance in 0.5 ~ 99.5 quantile.' % len(to_drop))
        return to_drop

    # def filter_correlation(self, feat_cols=None, threshold=0.98):
    #     if feat_cols is None:
    #         feat_cols = self.feats
    #
    #     print('Compute Corr Matrix ...')
    #     corr_matrix = self.df[feat_cols].corr().abs()
    #
    #     # Extract the upper triangle of the correlation matrix
    #     upper = pd.DataFrame(np.triu(corr_matrix, 1), feat_cols, feat_cols)
    #
    #     # Select the features with correlations above the threshold
    #     # Need to use the absolute value
    #     to_drop = [column for column in tqdm(upper.columns, 'Correlation Filter') if any(upper[column] > threshold)]
    #
    #     self.to_drop_correlation = to_drop
    #
    #     # Dataframe to hold correlated pairs
    #     # Iterate through the columns to drop to record pairs of correlated features
    #     corr_record = pd.DataFrame()
    #     for column in tqdm(to_drop, 'Correlation DataFrame'):
    #         cond = upper[column] > threshold
    #         corr_features = list(upper.index[cond])  # Find the correlated features
    #         corr_values = list(upper[column][cond])  # Find the correlated values
    #         drop_features = [column for _ in range(len(corr_features))]
    #         df_tmp = pd.DataFrame({'drop_feature': drop_features,
    #                                'corr_feature': corr_features,
    #                                'corr_value': corr_values})
    #         corr_record = corr_record.append(df_tmp, ignore_index=True, sort=False)
    #
    #     self.corr_record = corr_record
    #     print('%d features with a  correlation coefficient greater than %0.2f.' % (len(to_drop), threshold))
    #
    # @property
    # def to_drop_all(self):
    #     return self.to_drop_missing + self.to_drop_unique + self.to_drop_variance + self.to_drop_correlation + self.to_drop_zero_importance + self.to_drop_low_importance



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : illegal_char
# @Time         : 2019-09-03 17:36
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
import re

"""https://www.jianshu.com/p/4958bcdea12a"""


# def illegal_char(s):
#     s = re.compile(r"""^[\u4e00-\u9fa5\u0041-\u005A\u0061-\u007A\u0030-\u0039!@#$%^&*()-=[]{}\\;':",./<>?/*\+]+""") \
#         .sub('', s)
#
#     return s
#
# if __name__ == '__main__':
#     s = "《魔兽世界》官方宣布，为了维护玩家们的游戏体验，" \
#         "将为怀旧服高负载服务器的玩家们提供免费的角色定向转移服务，" \
#         "更多的详情将在晚些时候公布。官方也将在今晚18点新增两个PVP服务器阿什坎迪与怀特迈恩。 ​​​​"
#     print(illegal_char(s))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : __init__.py
# @Time         : 2019-06-20 11:14
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from ._in_notebook import _in_notebook
from .limit_memory import limit_memory
from .timer import timer
from .MyInfo import MyInfo
from .set_pandas import set_pandas
from .set_plot import set_plot



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '2019-06-03'
"""

import pysnooper


@pysnooper.snoop()
def number_to_bits(number):
    if number:
        bits = []
        while number:
            number, remainder = divmod(number, 2)
            bits.insert(0, remainder)
        return bits
    else:
        return [0]

import numpy as np

@pysnooper.snoop()
def f():
    return np.arange(100000).shape

if __name__ == '__main__':
    f()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'iter'
__author__ = 'JieYuan'
__mtime__ = '18-12-14'
"""
from .utils.xx import xx
from .utils import cprint
from .utils.config import _in_notebook

# cprint("Please Fork And Star:", 'black')
# print("\thttps://github.com/Jie-Yuan/tql-Python")

import warnings

warnings.filterwarnings("ignore")
if _in_notebook():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm
#########################################################################
import os
import json
import pickle
import inspect
import numpy as np
import pandas as pd
import jieba
import jieba.analyse as ja

from functools import reduce
from collections import Counter, OrderedDict
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor

#########################################################################


# p = Path(__file__)
get_module_path = lambda path, file=__file__: \
    os.path.normpath(os.path.join(os.getcwd(), os.path.dirname(file), path))

###################################################################


import seaborn as sns

sns.set(style="darkgrid")  # darkgrid, whitegrid, dark, white,和ticks
sns.set_context('paper')


# sns.plotting_context()
# sns.axes_style()
# plt.style.use('ggplot')


#########################################################################


# 序列化
# df.to_hdf('./data.h5', 'w', complib='blosc', complevel=8)
def reader(fname='./tmp.txt', sep=',', mode='r'):
    with open(fname, mode) as f:
        for l in f:
            yield l.strip().split(sep)


@xx
def xwrite(iterable, fname, mode='w', glue='\n'):
    with open(fname, mode) as f:
        for item in iterable:
            f.write(str(item) + glue)


@xx
def xpickle_dump(obj, file='tmp.pkl'):
    with open(file, 'wb') as f:
        pickle.dump(obj, f)


@xx
def xpickle_load(file):
    with open(file, 'rb') as f:
        return pickle.load(f)


# 统计函数: 待补充groupby.agg
# xsummary = xx(lambda iterable: DataFrameSummary(list(iterable) | xDataframe)['iterable'])
xvalue_counts = xx(
    lambda iterable, normalize=False, bins=None: pd.value_counts(list(iterable), normalize=normalize, bins=bins))

__funcs = [sum, min, max, abs, len, np.mean, np.median]
xsum, xmin, xmax, xabs, xlen, xmean, xmedian = [xx(i) for i in __funcs]

xnorm = xx(lambda iterable, ord=2: np.linalg.norm(iterable, ord))
xl1 = xx(lambda iterable: np.linalg.norm(iterable, 1))
xl2 = xx(lambda iterable: np.linalg.norm(iterable, 2))

xcount = xx(lambda iterable: Counter(list(iterable)))

xunique = xx(lambda iterable: list(OrderedDict.fromkeys(list(iterable))))  # 移除列表中的重复元素(保持有序)
xsort = xx(lambda iterable, reverse=False, key=None: sorted(list(iterable), key=key, reverse=reverse))

xmax_index = xx(lambda x: max(range(len(x)), key=x.__getitem__))  # 列表中最小和最大值的索引
xmin_index = xx(lambda x: min(range(len(x)), key=x.__getitem__))  # 列表中最小和最大值的索引
xmost_freq = xx(lambda x: max(set(x), key=x.count))  # 查找列表中频率最高的值, key作用于set(x), 可类推出其他用法


# print
@xx
def xprint(obj, mode=None, bg='blue'):
    if mode:
        for i in obj:
            cprint(i)
            print('\n')
    else:
        cprint(obj, bg)


xtqdm = xx(lambda iterable, desc=None: tqdm(iterable, desc))

# base types
xtuple, xlist, xset = xx(tuple), xx(list), xx(set)

# string
xjoin = xx(lambda s, sep=' ': sep.join(s))
xcut = xx(lambda s, cut_all=False: jieba.lcut(s, cut_all=cut_all))
xtfidf = xx(lambda s, topK=20: ja.tfidf(s, topK=topK))

# list transform
xgroup_by_step = xx(lambda ls, step=3: [ls[idx: idx + step] for idx in range(0, len(ls), step)])


# dict
@xx
def xjson(dict_):
    _ = json.dumps(dict_, default=lambda obj: obj.__dict__, sort_keys=True, indent=4)
    return _


@xx
def xSeries(iterable, name='iterable'):
    if isinstance(iterable, pd.Series):
        return iterable
    else:
        return pd.Series(iterable, name=name)


@xx
def xDataframe(iterable, name='iterable'):
    if isinstance(iterable, pd.DataFrame):
        return iterable
    else:
        return pd.DataFrame({name: iterable})


# 高阶函数
xmap = xx(lambda iterable, func: map(func, iterable))
xreduce = xx(lambda iterable, func: reduce(func, iterable))
xfilter = xx(lambda iterable, func: filter(func, iterable))


# multiple
@xx
def xThreadPoolExecutor(iterable, func, max_workers=5):
    """
    with ThreadPoolExecutor(max_workers) as pool:
        pool.map(func, iterable)
    """
    with ThreadPoolExecutor(max_workers) as pool:
        return pool.map(func, iterable)


@xx
def xProcessPoolExecutor(iterable, func, max_workers=5):
    """
    with ProcessPoolExecutor(max_workers) as pool:
        pool.map(func, iterable)
    """
    with ProcessPoolExecutor(max_workers) as pool:
        return pool.map(func, iterable)


# args
get_args = lambda func: inspect.getfullargspec(func).args



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : textcnn
# @Time         : 2019-06-23 19:16
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

"""
https://m.imooc.com/article/30122读大数据
https://blog.csdn.net/lujiandong1/article/details/54869170
https://blog.csdn.net/foreseerwang/article/details/80572182
"""
import tensorflow as tf
from tql.pipe import *
from tql.algo_nlp.utils import Text2Sequence
from tql.algo_dl.keras.utils import DataIter
from tql.algo_dl.keras.models import TextCNN, TextFast

jieba.initialize()

df = pd.read_csv('../../../fds/data/sentiment.tsv.zip', '\t')
ts = Text2Sequence(num_words=None, maxlen=128, tokenizer=jieba.cut)
X = ts.fit_transform(df.text.astype(str))
y = df.label

cnn = TextCNN(len(ts.word2index) + 1, maxlen=128, embedding_size=64)(1)
cnn.compile('adam', 'binary_crossentropy', metrics=['accuracy', tf.metrics.AUC()])
cnn.fit_generator(DataIter(X, y, batch_size=128), epochs=10)
# cnn.fit(X, y, batch_size=128, epochs=10, validation_split=0.3)



import numpy as np
import math
from scipy.linalg import norm
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer

# 依赖包numpy、python-Levenshtein、scipy

# euclidean,欧式距离算法，传入参数为两个向量，返回值为欧式距离
def Euclidean(vec1, vec2):
    npvec1, npvec2 = np.array(vec1), np.array(vec2)
    return math.sqrt(((npvec1-npvec2)**2).sum())


# Manhattan_Distance,曼哈顿距离
def Manhattan(vec1, vec2):
    npvec1, npvec2 = np.array(vec1), np.array(vec2)
    return np.abs(npvec1-npvec2).sum()


# Chebyshev_Distance,切比雪夫距离
def Chebyshev(vec1, vec2):
    npvec1, npvec2 = np.array(vec1), np.array(vec2)
    return max(np.abs(npvec1-npvec2))


# MinkowskiDistance 闵可夫斯基距离，其实就是上面三种距离的集合，这里就不重复了。
def Minkowski(vec1, vec2, params):
    pass

# def Standardized_Euclidean(vec1, vec2, v):
#     from scipy import spatial
#     npvec = np.array([np.array(vec1), np.array(vec2)])
#     return spatial.distance.pdist(npvec, 'seuclidean', V=None)
# Standardized Euclidean distance,标准化欧氏距离
# 在对长方体区域进行聚类的时候，普通的距离无法满足要求。
# 按照普通的距离聚类出的大多是圆形的区域，这时候要采用标准的欧式距离。
# 参考  标准化欧式距离：http://blog.csdn.net/jinzhichaoshuiping/article/details/51019473

# 马氏距离，优点：去除量纲，而且可以排除变量之间的相关性的干扰
x1 = [3, 5, 2]
x2 = [4, 6, 2]
x3 = [6,7,2]
x4 = [10,1,7]
def mahalanobis(*args, v1=None, v2=None): # v1,v2包含在args中
    X = np.vstack(args).T
    S = np.cov(X)
    SI = np.linalg.inv(S)
    return np.dot(np.dot((v1 - v2).T, SI), (v1 - v2))
print(mahalanobis([x1,x2,x3,x4],v1=x1,v2=x2))

# Cosine，余弦夹角
# 机器学习中借用这一概念来衡量样本向量之间的差异。
def Cosine(vec1, vec2):
    npvec1, npvec2 = np.array(vec1), np.array(vec2)
    return npvec1.dot(npvec2)/(math.sqrt((npvec1**2).sum()) * math.sqrt((npvec2**2).sum()))


################################################################################################
#-------------------------------------文本相似度算法-------------------------------------------#

# 针对列表改写的编辑距离，在NLP领域中，计算两个文本的相似度，是基于句子中词和词之间的差异。
# 如果使用传统的编辑距离算法，则计算的为文本中字与字之间的编辑次数。这里根据编辑距离的思维，
# 将编辑距离中的处理字符串中的字符对象，变成处理list中每个元素
def Edit_distance_array(str_ary1, str_ary2):
    len_str_ary1 = len(str_ary1) + 1
    len_str_ary2 = len(str_ary2) + 1
    matrix = [0 for n in range(len_str_ary1 * len_str_ary2)]
    for i in range(len_str_ary1):
        matrix[i] = i
    for j in range(0, len(matrix), len_str_ary1):
        if j % len_str_ary1 == 0:
            matrix[j] = j // len_str_ary1
    for i in range(1, len_str_ary1):
        for j in range(1, len_str_ary2):
            if str_ary1[i-1] == str_ary2[j-1]:
                cost = 0
            else:
                cost = 1
            matrix[j*len_str_ary1+i] = min(matrix[(j-1)*len_str_ary1+i]+1, matrix[j*len_str_ary1+(i-1)]+1, matrix[(j-1)*len_str_ary1+(i-1)] + cost)
    distance = int(matrix[-1])
    similarity = 1-int(matrix[-1])/max(len(str_ary1), len(str_ary2))
    return {'Distance': distance, 'Similarity': similarity}

# 余弦相似度算法
# 算法一 基于句子的TF
def cosine_similarity_tf(s1, s2):
    """
    计算两个句子的TF余弦相似度
    :param s1:
    :param s2:
    :return:
    """
    vectorizer = CountVectorizer(tokenizer=lambda s: s.split())
    corpus = [s1, s2]
    vectors = vectorizer.fit_transform(corpus).toarray()
    return np.dot(vectors[0], vectors[1]) / (norm(vectors[0]) * norm(vectors[1]))

# 算法二 基于句子的TFIDF
def cosine_similarity_tfidf(s1, s2):
    """
    计算两个句子的TFIDF余弦相似度
    :param s1:
    :param s2:
    :return:
    """
    vectorizer = TfidfVectorizer(tokenizer=lambda s: s.split())
    corpus = [s1, s2]
    vectors = vectorizer.fit_transform(corpus).toarray()
    return np.dot(vectors[0], vectors[1]) / (norm(vectors[0]) * norm(vectors[1]))

# WMD距离
# https://blog.csdn.net/cht5600/article/details/53405315
from gensim.similarities import WmdSimilarity

# Jaccard距离
def jaccard_similarity(s1, s2):
    """
    计算两个句子的雅可比相似度
    :param s1:
    :param s2:
    :return:
    """
    vectorizer = CountVectorizer(tokenizer=lambda s: s.split())
    corpus = [s1, s2]
    vectors = vectorizer.fit_transform(corpus).toarray()
    numerator = np.sum(np.min(vectors, axis=0))
    denominator = np.sum(np.max(vectors, axis=0))
    return 1.0 * numerator / denominator



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'x'
__author__ = 'JieYuan'
__mtime__ = '18-12-14'
"""

import functools

class xx:
    """I am very like a linux pipe"""

    def __init__(self, function):
        self.function = function
        functools.update_wrapper(self, function)

    def __ror__(self, other):
        return self.function(other)

    def __call__(self, *args, **kwargs):
        return xx(lambda x: self.function(x, *args, **kwargs))


if __name__ == '__main__':
    @xx
    def xfunc1(x):
        _ = x.split()
        print(_)
        return _


    @xx
    def xfunc2(x):
        _ = '>>'.join(x)
        print(_)
        return _


    'I am very like a linux pipe' | xfunc1 | xfunc2



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'CustomDataset'
__author__ = 'JieYuan'
__mtime__ = '2019/4/17'
"""
import torch
from torch.utils.data import Dataset, DataLoader, TensorDataset


def loader(X, y, batch_size, shuffle=True):
    _ = DataLoader(TensorDataset(X, y), batch_size=batch_size, shuffle=shuffle)
    return _


class CustomDataset(Dataset):

    # Initialize your data, download, etc.
    def __init__(self, X, y=None):
        self.X = torch.from_numpy(X)
        self.y = y
        if y is not None:
            self.y = torch.from_numpy(y)

    def __getitem__(self, index):
        if self.y is not None:
            return self.X[index], self.y[index]
        else:
            return self.X[index]

    def __len__(self):
        return self.X.shape[0]

    @classmethod
    def load(cls, X, y=None, batch_size=32, shuffle=True, drop_last=False):
        loader = DataLoader(
            dataset=cls(X, y),
            batch_size=batch_size,
            shuffle=shuffle,
            drop_last=drop_last
        )
        return loader



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-3-18'
"""
import os

from .multi_read_csv import multi_read_csv
from .read2write import read2write

base_dir = os.path.dirname(os.path.realpath('__file__'))
get_module_path = lambda path, file: os.path.normpath(os.path.join(os.getcwd(), os.path.dirname(file), path))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : __init__.py
# @Time         : 2019-06-21 14:43
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from .TextFast import TextFast
from .TextCNN import TextCNN
from .TextRNN import TextRNN
from .TextRNNBi import TextRNNBi
from .TextRCNN import TextRCNN
from .TextRCNNVariant import TextRCNNVariant



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : setup
# @Time         : 2019-06-17 16:12
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import os
import time
from setuptools import find_packages, setup

# rename
package_name = 'tql'
project_name = 'tql-Python'
version = time.strftime("%Y.%m.%d.%H.%M.%S", time.localtime())

with open("README.md", encoding='utf-8') as f:
    long_description = f.read()


def get_requirements():
    _ = './requirements.txt'
    if os.path.isfile(_):
        with open(_, encoding='utf-8') as f:
            return f.read().split()


setup(
    name=package_name,
    version=version,
    url='https://github.com/Jie-Yuan/' + project_name,
    keywords=["tool wheel", "yuanjie", 'utils'],
    description=('description'),
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='JieYuan',
    author_email='313303303@qq.com',
    maintainer='JieYuan',
    maintainer_email='313303303@qq.com',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['*.*']},
    platforms=["all"],
    python_requires='>=3.5',
    classifiers=[

        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
    ],

    install_requires=get_requirements(),

    entry_points={'console_scripts': [
        'tql-cli=tql.utils.cli:cli'
    ]}
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'Optimizer'
__author__ = 'JieYuan'
__mtime__ = '19-3-18'
"""

from sklearn.model_selection import cross_val_score
from bayes_opt import BayesianOptimization
from sklearn import clone
import numpy as np
from functools import partial
from pprint import pprint

from sklearn.model_selection import StratifiedKFold, KFold


class Optimizer(object):

    def __init__(self, estimator=None, pbounds: dict = None):
        """缩小区间长度（边界）
         Notice how we transform between regular and log scale. While this
         is not technically necessary, it greatly improves the performance
         of the optimizer.
        """
        self.estimator = estimator  # sk
        self.pbounds = pbounds
        self.params_type = self._get_params_type(self.pbounds)
        self.scaled_pbounds, self.scaled_params = self._scaler(self.pbounds)
        pprint("Scaled pbounds: %s" % self.scaled_pbounds)

        pprint(self.scaled_pbounds)

    def fit(self, X, y, n_iter=5, feval=None, folds=StratifiedKFold(5, True, 666), oof=True, seed=2019):

        _objective = partial(self.objective, X=X, y=y, oof=oof, feval=feval, folds=folds)
        self.optimizer = BayesianOptimization(
            f=_objective,
            pbounds=self.scaled_pbounds,
            random_state=seed,
            verbose=2
        )

        # gp_params = {"alpha": 1e-5, "n_restarts_optimizer": 3}
        self.optimizer.maximize(n_iter=n_iter)  # self.optimizer.maximize() 可以接着上一次优化继续下一轮优化
        result = self.optimizer.max.copy()
        result['params'] = self._scaler_reverse(result['params'])
        self.best_params = result['params']
        return result

    def objective(self, X, y, oof=True, feval=None, folds=None, **params):
        """cv_score: 核心函数"""
        params = self._scaler_reverse(params)
        estimator = clone(self.estimator)
        estimator.set_params(**params)
        if oof:
            oof = OOF(estimator, folds, verbose=1000)
            oof.fit(X, y, X[:1000], feval)
            cv_score = oof.score
        else:
            cv_score = cross_val_score(estimator, X, y, scoring='roc_auc', cv=5).mean()  # TODO 分数自定义
        return cv_score

    def _get_params_type(self, pbounds):
        _pbounds = pbounds.copy()

        params_type = {}
        for k, v in _pbounds.items():
            params_type.setdefault(str(type(v[0]))[8:-2], []).append(k)
        return params_type

    def _scaler(self, pbounds):
        scaled_params = []
        for k, v in pbounds.items():
            if not -1 <= np.log10(np.ptp(v)) <= 1:  # update pbounds
                scaled_params.append(k)
                pbounds[k] = np.log10(v[0] if v[0] else 1e-6), np.log10(v[1])
        _pbounds = pbounds.copy()

        scaled_params = {'float': [], 'int': [], 'str': []}
        for ps_type, ps in self.params_type.items():
            if ps_type == 'float':
                for p in ps:
                    v = _pbounds[p]
                    if np.ptp(v) / 10 ** -len(str(v[0]).split('.')[1]) > 100:  # 放缩阈值
                        scaled_params['float'].append(p)
                        _pbounds[p] = tuple(np.log10(v))

            if ps_type == 'int':
                for p in ps:
                    v = _pbounds[p]
                    if np.ptp(v) > 256:  # 放缩阈值
                        scaled_params['int'].append(p)
                        _pbounds[p] = tuple(np.log10(v))

            if ps_type == 'str':
                for p in ps:
                    v = _pbounds[p]
                    scaled_params['str'].append(p)
                    _pbounds[p] = (0, len(v) - 1)

        return _pbounds, scaled_params

    def _scaler_reverse(self, params):
        for p in self.scaled_params['float']:
            params[p] = 10 ** params[p]

        for p in self.scaled_params['int']:
            params[p] = int(10 ** params[p])
        for p in self.params_type['int']:
            params[p] = int(params[p])

        for p in self.scaled_params['str']:
            params[p] = self.pbounds[p][int(round(params[p]))]
        return params



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : FM
# @Time         : 2019-07-16 16:12
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


"""
https://github.com/jfpuget/LibFM_in_Keras/blob/master/keras_blog.ipynb
Trick:
1. 单模型
2. 提取embedding共享特征与原特征拼接 + 其他基模型
"""
from tensorflow.python.keras import backend as K
from tensorflow.python.keras.models import Model
from tensorflow.python.keras.layers.normalization import BatchNormalization
from tensorflow.python.keras.layers import Input, Embedding, Dense, Flatten, Concatenate, Dot, Reshape, Add, Subtract
from tensorflow.python.keras.optimizers import Adam
from tensorflow.python.keras.regularizers import l2
from tensorflow.python.keras.callbacks import EarlyStopping, ModelCheckpoint

k_latent = 2
embedding_reg = 0.0002
kernel_reg = 0.1


def get_embed(x_input, x_size, k_latent):
    if x_size > 0:  # category
        embed = Embedding(x_size, k_latent, input_length=1,
                          embeddings_regularizer=l2(embedding_reg))(x_input)
        embed = Flatten()(embed)
    else:
        embed = Dense(k_latent, kernel_regularizer=l2(embedding_reg))(x_input)
    return embed


def build_model_1(f_size):
    dim_input = len(f_size)

    input_x = [Input(shape=(1,)) for i in range(dim_input)]
    biases = [get_embed(x, size, 1) for (x, size) in zip(input_x, f_size)]
    factors = [get_embed(x, size, k_latent) for (x, size) in zip(input_x, f_size)]

    s = Add()(factors)
    diffs = [Subtract()([s, x]) for x in factors]
    dots = [Dot(axes=1)([d, x]) for d, x in zip(diffs, factors)]

    x = Concatenate()(biases + dots)
    x = BatchNormalization()(x)
    output = Dense(1, activation='relu', kernel_regularizer=l2(kernel_reg))(x)
    model = Model(inputs=input_x, outputs=[output])
    model.compile(optimizer=Adam(clipnorm=0.5), loss='mean_squared_error')
    output_f = factors + biases
    model_features = Model(inputs=input_x, outputs=output_f)
    return model, model_features



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'xgb'
__author__ = 'JieYuan'
__mtime__ = '19-1-2'
"""

import xgboost as xgb


class   BaselineXGB(object):
    """
    待补充: https://xgboost.readthedocs.io/en/release_0.81/tutorials/feature_interaction_constraint.html
    新版xgb支持交叉特征interaction_constraints
    tree_method='exact'

    b_xgb = BaselineXGB(X, y, learning_rate=0.01)
    b_xgb.run()
    """

    def __init__(self, X, y, learning_rate=0.01, missing=None, metrics='auc', feval=None, objective='binary:logistic',
                 scale_pos_weight=1, n_jobs=8, seed=0):  # seed不能为None
        """
        https://blog.csdn.net/fuqiuai/article/details/79495910
        https://blog.csdn.net/fantacy10000/article/details/84504394
        :param objective:
            'binary:logistic', 'multi:softmax', 'reg:linear'
        :param metrics: string, list of strings or None, optional (default=None)
            binary: 'auc', 'binary_error', 'binary_logloss'
            multiclass: 'multi_error', 'multi_logloss'
            https://lightgbm.readthedocs.io/en/latest/Parameters.html#metric-parameters
        :param feval:
            def feval(y_pred, y_true):
                y_true = y_true.get_label()
                return '1 / (1 + rmse)', 1 /(rmse(y_true, y_pred) + 1), True
        :param scale_pos_weight:
        """
        self.data = xgb.DMatrix(X, y, missing=missing)
        self.objective = objective
        self.metrics = metrics
        self.feval = feval
        self.best_iter = None
        # sklearn params
        self.params_sk = dict(
            booster='gbtree',
            objective=objective,
            max_depth=7,
            learning_rate=learning_rate,

            gamma=0.0,  # 描述分裂的最小 gain, 控制树的有用的分裂
            min_child_weight=1,  # 决定最小叶子节点样本权重和,使一个结点分裂的最小权值之和, 避免过拟合

            subsample=0.8,
            colsample_bytree=0.8,  # 每棵树的列数
            colsample_bylevel=0.8,  # 每一层的列数

            reg_alpha=0.0,
            reg_lambda=0.0,

            scale_pos_weight=scale_pos_weight,

            random_state=seed,
            n_jobs=n_jobs,
            silent=True
        )
        self.params = self.params_sk.copy()

        self.params['eta'] = self.params.pop('learning_rate')
        self.params['alpha'] = self.params.pop('reg_alpha')
        self.params['lambda'] = self.params.pop('reg_lambda')

        if self.objective == 'multi:softmax':
            self.num_class = len(set(y))
            self.params['objective'] = self.objective
            self.params['num_class'] = self.num_class

    def run(self, return_model=False, nfold=5, early_stopping_rounds=100, verbose_eval=50):

        print("XGB CV ...\n")
        try:
            cv_rst = xgb.cv(
                self.params,
                self.data,
                metrics=self.metrics,
                feval=self.feval,
                nfold=nfold,
                num_boost_round=2500,
                stratified=False if 'reg' in self.objective else True,
                early_stopping_rounds=early_stopping_rounds,
                verbose_eval=verbose_eval,
                as_pandas=False
            )
        except TypeError:
            print("Please: self.xgb_data = xgb.DMatrix(data, label=None, missing=None, feature_types=None)")

        if isinstance(self.metrics, str):
            _ = cv_rst['test-%s-mean' % self.metrics]
            self.best_iter = len(_)
            print('\nBest Iter: %s' % self.best_iter)
            print('Best Score: %s ' % _[-1])
        else:
            _ = cv_rst['test-%s-mean' % self.metrics]
            self.best_iter = len(_)
            print('\nBest Iter: %s' % self.best_iter)
            print('Best Score: %s ' % _[-1])

        self.params_sk['n_estimators'] = self.best_iter

        if return_model:
            print("\nReturning Model ...\n")
            return xgb.train(self.params, self.data, self.best_iter)



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import smtplib, time

# QQ邮箱

HOST = 'smtp.qq.com'  # 服务器主机,相当于第三方客户端

PORT = '465'  # 端口 使用SSL，端口号465或587

FROM = '发件人邮箱'  # 发件人的邮箱账号,必须是本人可登录的.
TO = ['yuanjie@xiaomi.com', '邮箱账号二']  # 接收邮件的人的账号.可以是类表;也可以是用,连接的字符串 '邮箱账号1', '邮箱账号二'

SUBJECT = '这是一封测试邮件'  # 邮件的标题
CONTENT = '这是一封邮件'  # 邮件的内容

# 创建邮件发送对象
smtp_obj1 = smtplib.SMTP()  # 普通邮件的发送形式
smtp_obj = smtplib.SMTP_SSL()  # 数据在传输过程中会被加密。
smtp_obj.connect(host=HOST, port=PORT)  # 需要进行发件人的认证，授权,smtp_obj就是一个第三方客户端对象
res = smtp_obj.login(user=FROM, password='授权码')  # 如果使用第三方客户端登录，要求使用授权码，不能使用真实密码，防止密码泄露。

print(res, '登录成功')

for to in TO:
    print(to)
    msg = '\n'.join(['From: {}'.format(FROM), 'To: {}'.format(to), 'SUBJECT:{}'.format(SUBJECT), '',
                     CONTENT])  # 发送邮件,这里是我们按照邮箱的格式拼接一下.
    for num in range(1, 5):  # 循环发送
        time.sleep(5)  # 设置一下睡眠时间
        smtp_obj.sendmail(from_addr=FROM, to_addrs=[to], msg=msg.encode('utf-8'))  # 这里要编码,不然会出现ASCII码编码错误.
        print('发送成功')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : textcnn_fasttext
# @Time         : 2019-06-23 19:10
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from tql.pipe import *
from tql.algo_nlp.utils import Text2SequenceByFastText
from tql.algo_dl.keras.utils import DataIter
from tql.algo_dl.keras.models import TextCNN
from gensim.models.fasttext import load_facebook_model

jieba.initialize()

fasttext = load_facebook_model('../../../fds/data/wv/skipgram.title')
df = pd.read_csv('../../../fds/data/sentiment.tsv.zip', '\t')
ts = Text2SequenceByFastText(maxlen=128, fasttext_model=fasttext, tokenizer=jieba.cut)
X = ts.fit_transform(df.text.astype(str))
y = df.label

cnn = TextCNN(len(ts.word2index), maxlen=128, weights=ts.weights)(1)
cnn.compile('adam', 'binary_crossentropy', metrics=['accuracy'])
cnn.fit_generator(DataIter(X, y, batch_size=128), epochs=10)
# cnn.fit(X, y, batch_size=128, epochs=10, validation_split=0.3)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '2019/4/17'
"""
import torch

from .ConfigTorch import TorchConfig
from .CustomDataset import CustomDataset
from .WeightInit import WeightInit


#################################################################

def torch_save(model_cls, model_pt):
    torch.save(model_cls.state_dict(), model_pt)


def torch_load(model_cls, model_pt):
    model_cls.load_state_dict(torch.load(model_pt))


# 类绑定方法
# https://www.cnblogs.com/seirios1993/p/6624157.html
class Model:

    @classmethod
    def load(cls, params_path='params.ckpt'):
        model = cls()
        model.load_state_dict(torch.load(params_path))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'TODO'
__author__ = 'JieYuan'
__mtime__ = '2019-04-23'
"""
import pandas as pd

df = pd.DataFrame()
# 分析两个类别型特征得到新特征rate
df.groupby('author')['category'] \
    .agg({'rate': lambda x: pd.value_counts(x, normalize=True)[:1]}) \
    .reset_index()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'debug_info'
__author__ = 'JieYuan'
__mtime__ = '19-1-29'
"""
from datetime import datetime

"""强大的日志模块loguru"""


def debug_info(message='message', level=0):
    _level = {
        0: 'INFO',
        1: 'WARNING',
        2: 'DEBUG'
    }
    return f"{datetime.now()} | {_level[level]}: {message}"



# -*- coding: utf-8 -*-
"""
__title__ = 'Singleton'
__author__ = 'JieYuan'
__mtime__ = '2018/7/28'
"""
import functools
import inspect
import time
from collections import Iterable

import numpy as np
import wrapt
from tqdm import tqdm

"""定义装饰器"""


@wrapt.decorator
def pass_through(wrapped, instance, args, kwargs):
    return wrapped(*args, **kwargs)


def with_arguments(myarg1, myarg2):
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        return wrapped(*args, **kwargs)

    return wrapper


@wrapt.decorator
def universal(wrapped, instance, args, kwargs):
    """
    class A:
        @universal
        def f(self, x):
            return x

    @universal
    def f(x):
        return x
    """
    if instance is None:
        if inspect.isclass(wrapped):
            print('Decorator was applied to a class.')
            return wrapped(*args, **kwargs)
        else:
            print('Decorator was applied to a function or staticmethod.')
            return wrapped(*args, **kwargs)
    else:
        if inspect.isclass(instance):
            print('Decorator was applied to a classmethod.')
            return wrapped(*args, **kwargs)
        else:
            print('Decorator was applied to an instancemethod.')
            return wrapped(*args, **kwargs)


##################################################################################

##################################################################################
def mytqdm(desc=None):
    def wrapper(func):
        @functools.wraps(func)
        def _wrapper(iter_obj, *args, **kwargs):
            """保证第一个参数为可迭代参数即可"""
            assert isinstance(iter_obj, Iterable)
            with tqdm(iter_obj, desc) as t:
                temp = func(t, *args, **kwargs)
            return temp

        return _wrapper

    return wrapper


"""类里调用增加self参数使得能够修饰类方法
def mytqdm(desc=None):
    def wrapper(func):
        @functools.wraps(func)
        def _wrapper(_self, iter_obj, *args, **kwargs):
            assert isinstance(iter_obj, Iterable)
            with tqdm(iter_obj, desc) as t:
                temp = func(_self, t, *args, **kwargs)
            return temp

        return _wrapper

    return wrapper
"""


##################################################################################
def feval(multiclass=None, is_bigger_better=True, model='lgb'):
    """example
    @feval(3)
    def f1_score(y_pred, y_true):
        '注意入参顺序'
        return f1_score(y_true, y_pred, average='macro')
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        y_pred, y_true = args
        y_true = y_true.get_label()
        if model == 'lgb':
            if multiclass:
                y_pred = np.array(y_pred).reshape(multiclass, -1).argmax(0)
            return wrapped.__name__, wrapped(y_pred, y_true), is_bigger_better
        elif is_bigger_better:
            """xgb评估指标默认越小越好"""
            return '-' + wrapped.__name__, - wrapped(y_pred, y_true)
        else:
            return wrapped.__name__, wrapped(y_pred, y_true)

    return wrapper


##################################################################################
@wrapt.decorator
def execution_time(wrapped, instance, args, kwargs):
    def call():
        start = time.time()
        tmp = wrapped(*args, **kwargs)
        print("""\nExecuting Time of "%s": %.3f s""" % (wrapped.__name__, time.time() - start))
        return tmp

    if instance is None:
        if inspect.isclass(wrapped):
            print('Decorator was applied to a class.')
            return call()
        else:
            print('Decorator was applied to a function or staticmethod.')
            return call()
    else:
        if inspect.isclass(instance):
            print('Decorator was applied to a classmethod.')
            return call()
        else:
            print('Decorator was applied to an instancemethod.')
            return call()


##################################################################################
class Singleton:
    def __init__(self, cls):
        self.cls = cls
        self.instance = None

    def __call__(self, *args, **kwargs):
        if self.instance is None:
            self.instance = self.cls(*args, **kwargs)
        return self.instance


def singleton(cls):
    """单例"""
    instances = {}

    def getinstance(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return getinstance



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : DataGenerator
# @Time         : 2019-06-20 17:22
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :


from sklearn.utils import shuffle


class DataGenerator(object):
    """
    https://blog.csdn.net/leviopku/article/details/87912097
    https://github.com/afshinea/keras-data-generator
    https://blog.csdn.net/u011311291/article/details/80991330
    """

    def __init__(self, X, y, batch_size=32, mapper=lambda *args: args):
        """

        :param X:
        :param y:
        :param batch_size:
        :param maxlen:
        :param is_train:
        :param mapper:
            # bert keras
            from keras.preprocessing.sequence import pad_sequences
            from keras_bert import load_trained_model_from_checkpoint, Tokenizer

            def mapper(X, y):
                X = list(map(lambda x: pad_sequences(x, 128), zip(*map(tokenizer.encode, X))))
                return X, y

        """
        self.X = X
        self.y = y
        self.batch_size = batch_size
        self.steps = (len(X) + batch_size - 1) // batch_size

        assert callable(mapper) is True
        self.mapper = mapper

    def __len__(self):
        return self.steps

    def __iter__(self):
        self.X, self.y = shuffle(self.X, self.y)
        while 1:
            for i in range(self.steps):
                idx_s = i * self.batch_size
                idx_e = (i + 1) * self.batch_size

                X = self.X[idx_s:idx_e]
                y = self.y[idx_s:idx_e]

                yield self.mapper(X, y)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : BaseOOF
# @Time         : 2019-06-23 20:07
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import time
import numpy as np
import pandas as pd
from sklearn.model_selection import StratifiedKFold, RepeatedStratifiedKFold, KFold, cross_val_predict, cross_validate

from catboost import CatBoostClassifier
from sklearn.metrics import roc_auc_score

import os

cloudml = os.path.exists('/fds')


class CatBoostClassifierCV(object):
    """cross_val_predict"""

    def __init__(self, params=None, cv=5, random_state=None, n_repeats=None):
        self.clf = CatBoostClassifier()
        if params:
            self.clf.set_params(**params)
        if n_repeats:
            self._kf = RepeatedStratifiedKFold(cv, True, random_state)
            self._num_preds = cv * n_repeats
        else:
            self._kf = StratifiedKFold(cv, True, random_state)
            self._num_preds = cv

    def fit(self, X, y, X_test, feval=roc_auc_score, cat_features=None, sample_weight=None, verbose=100,
            early_stopping_rounds=100, plot=False, silent=None,
            logging_level=None, column_description=None, save_snapshot=None,
            snapshot_file='/fds/data' if cloudml else None, snapshot_interval=None,
            init_model=None):
        """输入数组"""

        self.oof_train = np.zeros(len(X))
        self.oof_test = np.zeros((len(X_test), self._num_preds))
        for n_fold, (train_index, valid_index) in enumerate(self._kf.split(X, y)):
            if verbose:
                print("\033[94mFold %s started at %s\033[0m" % (n_fold + 1, time.ctime()))
            X_train, y_train = X[train_index], y[train_index]
            X_valid, y_valid = X[valid_index], y[valid_index]
            eval_set = [(X_train, y_train), (X_valid, y_valid)]

            ########################################################################
            self.clf.fit(X_train, y_train,
                         cat_features=cat_features,
                         sample_weight=sample_weight,
                         use_best_model=True,
                         eval_set=eval_set,
                         verbose=verbose,
                         logging_level=logging_level,
                         plot=plot,
                         column_description=column_description,
                         silent=silent,
                         early_stopping_rounds=early_stopping_rounds,
                         save_snapshot=save_snapshot,
                         snapshot_file=snapshot_file,
                         snapshot_interval=snapshot_interval,
                         init_model=init_model)

            self.oof_train[valid_index] = self.clf.predict_proba(X_valid)[:, 1]
            self.oof_test[:, n_fold] = self.clf.predict_proba(X_test)[:, 1]
            ########################################################################

        # 输出 测试集 oof
        self.oof_test_rank = pd.DataFrame(self.oof_test).rank().mean(1) / len(self.oof_test)
        self.oof_test = self.oof_test.mean(1)

        # 计算 训练集 oof 得分
        if feval:
            score = feval(y, self.oof_train)
            print(f"\n\033[94mCV Score: {score} ended at {time.ctime()}\033[0m")
            return score

    def oof_save(self, file='./oof_train_and_test.csv'):
        assert isinstance(file, str)
        _ = np.append(self.oof_train, self.oof_test)
        pd.DataFrame(_, columns='oof_train_and_test').to_csv(file, index=False)


if __name__ == "__main__":
    from sklearn.datasets import make_classification

    X, y = make_classification()
    X_test, _ = make_classification()

    clf = CatBoostClassifierCV({'n_estimators': 100, 'eval_metric': 'AUC'})
    clf.fit(X, y, X_test, verbose=50, plot=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'split2cols'
__author__ = 'JieYuan'
__mtime__ = '19-3-12'
"""
import pandas as pd

s = pd.Series([[1, 2, 3], [4, 5, 6]])

s.tolist()


def split2cols(series: pd.Series, columns=None):
    return pd.DataFrame(series.tolist(), columns=columns)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : featureSelector
# @Time         : 2019-07-26 13:39
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from mlxtend.feature_selection import SequentialFeatureSelector as SFS
from mlxtend.plotting import plot_sequential_feature_selection as plot_sfs
from sklearn.neighbors import NearestNeighbors
from sklearn.feature_selection import GenericUnivariateSelect, \
    SelectPercentile, SelectKBest, f_classif, mutual_info_classif, RFE

from lightgbm import LGBMClassifier
import matplotlib.pyplot as plt

sfs = SFS(LGBMClassifier(),
          k_features=10,
          forward=True,
          floating=False,
          verbose=2,
          scoring='roc_auc',
          cv=5,
          n_jobs=-1)

sfs.fit(X, y)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'DataLoader'
__author__ = 'JieYuan'
__mtime__ = '19-3-22'
"""
import tensorflow as tf
import pandas as pd


class DataLoader(object):

    def __init__(self, X: pd.DataFrame, y=None, batch_size=128):
        self.X = dict(X)
        self.y = y
        self.batch_size = batch_size

    def train_input_fn(self):
        """An input function for training
        https://www.tensorflow.org/alpha/tutorials/estimators/boosted_trees
        """
        inputs = (self.X, self.y)

        # Convert the inputs to a Dataset.
        dataset = tf.data.Dataset.from_tensor_slices(inputs)

        # Shuffle, repeat, and batch the examples.
        dataset = dataset.shuffle(1000).repeat().batch(self.batch_size)

        # Return the dataset.
        return dataset

    def eval_input_fn(self):
        """An input function for evaluation or prediction"""
        if self.y is None:
            # No labels, use only features.
            inputs = self.X
        else:
            inputs = (self.X, self.y)

        # Convert the inputs to a Dataset.
        dataset = tf.data.Dataset.from_tensor_slices(inputs)

        # Batch the examples
        dataset = dataset.batch(self.batch_size)

        # Return the dataset.
        return dataset


if __name__ == '__main__':
    from tensorflow_estimator import estimator
    from sklearn.datasets import load_iris

    tf.get_logger().setLevel(2)

    CSV_COLUMN_NAMES = ['SepalLength', 'SepalWidth',
                        'PetalLength', 'PetalWidth']  # 'Species'
    X, y = load_iris(True)
    X = pd.DataFrame(X, columns=CSV_COLUMN_NAMES)

    # Feature columns describe how to use the input.
    my_feature_columns = list(map(tf.feature_column.numeric_column, CSV_COLUMN_NAMES))

    clf = estimator.DNNClassifier(hidden_units=[10, 10],
                                  feature_columns=my_feature_columns,
                                  n_classes=3,
                                  batch_norm=True)

    clf = estimator.BoostedTreesClassifier(feature_columns=my_feature_columns,
                                           n_batches_per_layer=2,
                                           n_classes=2,
                                           n_trees=100,
                                           max_depth=6,
                                           learning_rate=0.1,
                                           l1_regularization=0.,
                                           l2_regularization=0.,
                                           tree_complexity=0.,
                                           min_node_weight=0.,
                                           center_bias=False,
                                           pruning_mode='none',
                                           quantile_sketch_epsilon=0.01)

    clf.train(DataLoader(X[:100], y[:100], 64).train_input_fn, steps=1000)

    predict_x = {
        'SepalLength': [5.1, 5.9, 6.9],
        'SepalWidth': [3.3, 3.0, 3.1],
        'PetalLength': [1.7, 4.2, 5.4],
        'PetalWidth': [0.5, 1.5, 2.1],
    }

    print(list(clf.predict(DataLoader(predict_x).eval_input_fn)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-1-30'
"""
from .WordCloud import WordCloud
from .chinese_setting import chinese_setting

import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns


def plot_feature_importances(features, importances, topk=20, figsize=(10, 6)):
    columns = ['Importances', 'Features']
    df = pd.DataFrame(list(zip(importances, features)), columns=columns).sort_values('Importances', 0, False)

    plt.figure(figsize=figsize)
    sns.barplot(*columns, data=df[:topk])
    plt.title('Features Importances\n')
    plt.tight_layout()
    plt.savefig('lgbm_importances.png')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'rank_encoder'
__author__ = 'JieYuan'
__mtime__ = '19-3-11'
"""
from collections import OrderedDict

import pandas as pd
from sklearn.base import BaseEstimator, TransformerMixin


class RankEncoder(BaseEstimator, TransformerMixin):

    def __init__(self, topn=None, verbose=True):
        self.topn = (topn if topn else 10000)
        self.verbose = verbose
        self.mapper = None

    def fit(self, series: pd.Series):
        _ = series.value_counts(True)[:self.topn]
        print("Coverage: %.2f %%" % (_.sum() * 100))
        self.mapper = (_.rank(method='first') - 1).to_dict(OrderedDict)
        if self.verbose:
            print(self.mapper)
        return self

    def transform(self, series: pd.Series):
        return series.map(self.mapper).fillna(0)  # 不在训练集里的补0


if __name__ == '__main__':
    s = pd.Series(['a', 'a', 'b', 'b', 'c'])

    re = RankEncoder()

    print(re.fit_transform(s))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : json2cls
# @Time         : 2019-09-03 12:36
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


class CreateOrder(object):
    def __init__(self):
        self.success = None
        self.item = None


co = CreateOrder()
co.__dict__ = {'success': True,
               'item': {'id': '1f4652c2f841f94f77b29a7836ffb971',
                        'title': '“乱港分子”黄之锋抵台 将拜会民进党',
                        'url': 'https://mb.yidianzixun.com/article/0N7BdC2A?s=mb&appid=mibrowser&miref=newsin_push_model',
                        'category': ['时政'],
                        'publishTime': '2019-09-03 11:03:02',
                        'keywords': ['时政', '香港', '台湾'],
                        'source': '快讯',
                        'valid': True,
                        'userTags': ['时政', '香港', '台湾'],
                        'crawlTime': 1567479784181,
                        'cpApi': 'cn-browser-push',
                        'isReservedDoc': True,
                        'updateTime': 1567479784181,
                        'bizIndexName': 'BROWSER_NEWS'}}

print(co.success)
print(co.item)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : iWork.
# @File         : demo
# @Time         : 2019-06-14 16:46
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

# TODO: 爬虫服务
from selenium import webdriver

options = webdriver.ChromeOptions()
# options.add_argument('--no-sandbox')
# options.add_argument('--disable-extensions')
# options.add_argument('--headless')
# options.add_argument('--disable-gpu')
# options.add_argument('blink-settings=imagesEnabled=false')
# options.add_argument('user-agent="Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.100 Safari/537.36"')
options.add_argument("--proxy-server=http://proxy.pt.xiaomi.com:80")

driver = webdriver.Chrome(options=options)
url = "http://zjy-hadoop-prc-ct12.bj:20701"
driver.get(url)
# driver.page_source
# driver.implicitly_wait(10)


# url = "https://m.weibo.cn/"
# driver.get(url)
# d = driver.find_element_by_xpath('//*[@id="app"]/div[1]/div[1]/div[2]/div/div[1]/div/div/ul/li[9]/span')
#
# d.click()
# from lxml.etree import HTML
# dom_tree = HTML(driver.page_source)
# title = dom_tree.xpath('//div[@class="weibo-text"]//text()')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'TorchConfig'
__author__ = 'JieYuan'
__mtime__ = '2019/4/17'
"""

import os
import torch
import random
import numpy as np


class TorchConfig(object):
    """Hyper-parameters"""

    input_size = 784
    hidden_size = 500
    num_classes = 10

    num_epochs = 5
    batch_size = 128
    lr = 0.001

    def __init__(self, seed=2019):
        torch.cuda.empty_cache()
        self._seed = seed
        self.use_cuda = torch.cuda.is_available()
        self.device = torch.device('cuda' if self.use_cuda else 'cpu')  # Device configuration
        self.set_seed()
        # torch.set_default_tensor_type(torch.DoubleTensor)

    def set_seed(self):
        os.environ['PYTHONHASHSEED'] = '0'
        random.seed(self._seed)
        np.random.seed(self._seed)
        torch.manual_seed(self._seed)
        if self.use_cuda:
            print('GPU: %s' % torch.cuda.get_device_name(0))
            torch.cuda.manual_seed(self._seed)
            # torch.backends.cudnn.benchmark = True # Benchmark 模式会提升计算速度，但是由于计算中有随机性，每次网络前馈结果略有差异。
            torch.backends.cudnn.deterministic = True  # 避免这种结果波动



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : iWork.
# @File         : cli
# @Time         : 2019-06-14 12:20
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import fire


# fire.Fire(lambda obj: type(obj).__name__)


class Calculator(object):
    """doc"""

    def __init__(self, arg=0, **kwargs):
        """

        :param arg:
        """
        self.arg = arg
        print(kwargs)

    def add(self, x, y, **kwargs):
        """add"""
        print(kwargs)
        return x + y

    def multiply(self, x, y):
        """add"""
        return x * y + self.arg

    def get_list(self, x):
        print(type(x))
        return x


def cli():
    fire.Fire(Calculator)


if __name__ == '__main__':
    # calculator = Calculator()
    fire.Fire(Calculator)

# 方法用 - 分割
"""
python cli.py --arg 0 --a 1 
- add 1 2 --b 1000
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'bayes_opt_demo'
__author__ = 'JieYuan'
__mtime__ = '19-3-18'
"""

from sklearn.datasets import make_classification
from sklearn.model_selection import cross_val_score
from sklearn.ensemble import RandomForestClassifier as RFC
from sklearn.svm import SVC

from bayes_opt import BayesianOptimization
from bayes_opt.util import Colours

def get_data():
    """Synthetic binary classification dataset."""
    data, targets = make_classification(
        n_samples=1000,
        n_features=45,
        n_informative=12,
        n_redundant=7,
        random_state=134985745,
    )
    return data, targets


def svc_cv(C, gamma, data, targets):
    """SVC cross validation.
    This function will instantiate a SVC classifier with parameters C and
    gamma. Combined with data and targets this will in turn be used to perform
    cross validation. The result of cross validation is returned.
    Our goal is to find combinations of C and gamma that maximizes the roc_auc
    metric.
    """
    estimator = SVC(C=C, gamma=gamma, random_state=2)
    cval = cross_val_score(estimator, data, targets, scoring='roc_auc', cv=4)
    return cval.mean()



def rfc_cv(n_estimators, min_samples_split, max_features, data, targets):
    """Random Forest cross validation.
    This function will instantiate a random forest classifier with parameters
    n_estimators, min_samples_split, and max_features. Combined with data and
    targets this will in turn be used to perform cross validation. The result
    of cross validation is returned.
    Our goal is to find combinations of n_estimators, min_samples_split, and
    max_features that minimzes the log loss.
    """
    estimator = RFC(
        n_estimators=n_estimators,
        min_samples_split=min_samples_split,
        max_features=max_features,
        random_state=2
    )
    cval = cross_val_score(estimator, data, targets,
                           scoring='neg_log_loss', cv=4)
    return cval.mean()


def optimize_svc(data, targets):
    """Apply Bayesian Optimization to SVC parameters."""

    def svc_crossval(expC, expGamma):
        """Wrapper of SVC cross validation.
        Notice how we transform between regular and log scale. While this
        is not technically necessary, it greatly improves the performance
        of the optimizer.
        """
        C = 10 ** expC
        gamma = 10 ** expGamma
        return svc_cv(C=C, gamma=gamma, data=data, targets=targets)

    optimizer = BayesianOptimization(
        f=svc_crossval,
        pbounds={"expC": (-3, 2), "expGamma": (-4, -1)},
        random_state=1234,
        verbose=2
    )
    optimizer.maximize(n_iter=10)

    print("Final result:", optimizer.max)


def optimize_rfc(data, targets):
    """Apply Bayesian Optimization to Random Forest parameters."""

    def rfc_crossval(n_estimators, min_samples_split, max_features):
        """Wrapper of RandomForest cross validation.
        Notice how we ensure n_estimators and min_samples_split are casted
        to integer before we pass them along. Moreover, to avoid max_features
        taking values outside the (0, 1) range, we also ensure it is capped
        accordingly.
        """
        return rfc_cv(
            n_estimators=int(n_estimators),
            min_samples_split=int(min_samples_split),
            max_features=max(min(max_features, 0.999), 1e-3),
            data=data,
            targets=targets,
        )

    optimizer = BayesianOptimization(
        f=rfc_crossval,
        pbounds={
            "n_estimators": (10, 250),
            "min_samples_split": (2, 25),
            "max_features": (0.1, 0.999),
        },
        random_state=1234,
        verbose=2
    )
    optimizer.maximize(n_iter=10)

    print("Final result:", optimizer.max)


if __name__ == "__main__":
    data, targets = get_data()

    print(Colours.yellow("--- Optimizing SVM ---"))
    optimize_svc(data, targets)

    print(Colours.green("--- Optimizing Random Forest ---"))
    optimize_rfc(data, targets)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'rf'
__author__ = 'JieYuan'
__mtime__ = '19-1-2'
"""

from sklearn.ensemble import ExtraTreesClassifier, RandomForestClassifier, GradientBoostingClassifier
from sklearn.linear_model import LogisticRegression


clf1 = ExtraTreesClassifier(
    256, n_jobs=8, class_weight='balanced', random_state=None)
clf2 = RandomForestClassifier(
    256,
    criterion='gini',
    n_jobs=8,
    class_weight='balanced',
    random_state=None)
clf3 = RandomForestClassifier(
    256,
    criterion='entropy',
    n_jobs=8,
    class_weight='balanced',
    random_state=None)
clf4 = GradientBoostingClassifier(
    'deviance',
    learning_rate=0.1,
    n_estimators=256,
    subsample=0.8,
    random_state=None)
clf5 = GradientBoostingClassifier(
    'exponential',
    learning_rate=0.1,
    n_estimators=256,
    subsample=0.8,
    random_state=None)

classifiers = [clf1, clf2, clf3, clf4, clf5]
meta_classifier = LogisticRegression()
if __name__ == '__main__':
    from mlxtend.classifier import StackingCVClassifier
    from sklearn.model_selection import StratifiedKFold

    sclf = StackingCVClassifier(
        classifiers,
        meta_classifier,
        use_probas=True,
        cv=StratifiedKFold(5, True),
        use_features_in_secondary=False,
        stratify=True,
        shuffle=True,
        verbose=2,
        store_train_meta_features=False,
        use_clones=True)



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'replace'
__author__ = 'JieYuan'
__mtime__ = '2019-05-06'
"""
def replace(s, dic):
    return s.translate(str.maketrans(dic))



if __name__ == '__main__':
    print(replace('abcd', {'a': '8', 'd': '88'}))


